# django imports
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.db import models
from django.utils.translation import ugettext_lazy as _


class CriteriaObjects(models.Model):
    """Assigns arbitrary criteria to arbitrary content objects.
    """
    class Meta:
        ordering = ["position"]
        verbose_name_plural = "Criteria objects"
        app_label = "criteria"

    criterion_type = models.ForeignKey(ContentType, verbose_name=_(u"Criterion type"), related_name="criterion")
    criterion_id = models.PositiveIntegerField(_(u"Content id"))
    criterion = generic.GenericForeignKey(ct_field="criterion_type", fk_field="criterion_id")

    content_type = models.ForeignKey(ContentType, verbose_name=_(u"Content type"), related_name="content_type")
    content_id = models.PositiveIntegerField(_(u"Content id"))
    content = generic.GenericForeignKey(ct_field="content_type", fk_field="content_id")

    position = models.PositiveIntegerField(_(u"Position"), default=999)



# django imports
from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import ugettext_lazy as _


class Supplier(models.Model):
    """
    * A supplier holds all shop supplier related information
    * A Supplier is only created by the system administrator
    """
    user = models.ForeignKey(User)
    name = models.CharField(max_length=100)
    slug = models.SlugField(_(u"Slug"), unique=True, max_length=80)
    active = models.BooleanField(default=True)

    def __unicode__(self):
        return "%s" % (self.name)



# django imports
from django.contrib.auth.decorators import permission_required
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils.translation import ugettext_lazy as _
from django.utils import simplejson

# lfs.imports
import lfs.core.utils
from lfs.caching.utils import lfs_get_object_or_404
from lfs.catalog.models import Image
from lfs.catalog.models import Product
from lfs.core.signals import product_changed
from lfs.core.utils import LazyEncoder

# Load logger
import logging
logger = logging.getLogger("default")

@permission_required("core.manage_shop", login_url="/login/")
def manage_images(request, product_id, as_string=False, template_name="manage/product/images.html"):
    """
    """
    product = lfs_get_object_or_404(Product, pk=product_id)

    result = render_to_string(template_name, RequestContext(request, {
        "product": product,
    }))

    if as_string:
        return result
    else:
        result = simplejson.dumps({
            "images": result,
            "message": _(u"Images has been added."),
        }, cls=LazyEncoder)

        return HttpResponse(result)


# Actions
# @permission_required("core.manage_shop", login_url="/login/")
def add_image(request, product_id):
    """Adds an image to product with passed product_id.
    """
    product = lfs_get_object_or_404(Product, pk=product_id)
    if request.method == "POST":
        for file_content in request.FILES.getlist("file"):
            image = Image(content=product, title=file_content.name)
            try:
                image.image.save(file_content.name, file_content, save=True)
            except Exception, e:
                logger.info("Upload image: %s %s" % (file_content.name, e))
                continue

    # Refresh positions
    for i, image in enumerate(product.images.all()):
        image.position = (i + 1) * 10
        image.save()

    product_changed.send(product, request=request)

    result = simplejson.dumps({"name": file_content.name, "type": "image/jpeg", "size": "123456789"})
    return HttpResponse(result)


@permission_required("core.manage_shop", login_url="/login/")
def update_images(request, product_id):
    """Saves/deletes images with given ids (passed by request body).
    """
    product = lfs_get_object_or_404(Product, pk=product_id)

    action = request.POST.get("action")
    if action == "delete":
        message = _(u"Images has been deleted.")
        for key in request.POST.keys():
            if key.startswith("delete-"):
                try:
                    id = key.split("-")[1]
                    image = Image.objects.get(pk=id).delete()
                except (IndexError, ObjectDoesNotExist):
                    pass

    elif action == "update":
        message = _(u"Images has been updated.")
        for key, value in request.POST.items():
            if key.startswith("title-"):
                id = key.split("-")[1]
                try:
                    image = Image.objects.get(pk=id)
                except ObjectDoesNotExist:
                    pass
                else:
                    image.title = value
                    image.save()

            elif key.startswith("position-"):
                try:
                    id = key.split("-")[1]
                    image = Image.objects.get(pk=id)
                except (IndexError, ObjectDoesNotExist):
                    pass
                else:
                    image.position = value
                    image.save()

    # Refresh positions
    for i, image in enumerate(product.images.all()):
        image.position = (i + 1) * 10
        image.save()

    product_changed.send(product, request=request)

    html = [["#images", manage_images(request, product_id, as_string=True)]]
    result = simplejson.dumps({
        "html": html,
        "message": message,
    }, cls=LazyEncoder)

    return HttpResponse(result)


@permission_required("core.manage_shop", login_url="/login/")
def move_image(request, id):
    """Moves the image with passed id up or down.

    **Parameters:**

        id
            The id of the image which should be edited.

    **Query String:**

        direction
            The direction in which the image should be moved. One of 0 (up)
            or 1 (down).

    **Permission:**

        edit (of the belonging content object)
    """
    image = Image.objects.get(pk=id)
    product = image.content

    direction = request.GET.get("direction", 0)

    if direction == "1":
        image.position += 15
    else:
        image.position -= 15
        if image.position < 0:
            image.position = 10

    image.save()

    # Refresh positions
    for i, image in enumerate(product.images.all()):
        image.position = (i + 1) * 10
        image.save()

    html = [["#images", manage_images(request, product.id, as_string=True)]]

    result = simplejson.dumps({
         "html": html,
    }, cls=LazyEncoder)

    return HttpResponse(result)


@permission_required("core.manage_shop", login_url="/login/")
def update_active_images(request, product_id):
    """Updates the images activity state for product variants.
    """
    product = Product.objects.get(pk=product_id)
    if request.POST.get("active_images"):
        product.active_images = True
    else:
        product.active_images = False
    product.save()

    return lfs.core.utils.set_message_cookie(
        url=reverse("lfs_manage_product", kwargs={"product_id": product.id}),
        msg=_(u"Active images has been updated."),
    )



# django imports
from django.contrib.auth.decorators import permission_required
from django.core.urlresolvers import reverse
from django.forms import ModelForm
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.http import require_POST

# lfs imports
import lfs.core.utils
from lfs.catalog.models import DeliveryTime
from lfs.catalog.models import Product


class DeliveryTimeAddForm(ModelForm):
    """Form to edit add a delivery time.
    """
    class Meta:
        model = DeliveryTime
        fields = ("min", "max", "unit")


class DeliveryTimeForm(ModelForm):
    """Form to edit a delivery time.
    """
    class Meta:
        model = DeliveryTime


@permission_required("core.manage_shop", login_url="/login/")
def manage_delivery_times(request):
    """Dispatches to the first delivery time or to the form to add a delivery
    time (if there is no delivery time yet).
    """
    try:
        delivery_time = DeliveryTime.objects.all()[0]
        url = reverse("lfs_manage_delivery_time", kwargs={"id": delivery_time.id})
    except IndexError:
        url = reverse("lfs_add_delivery_time")

    return HttpResponseRedirect(url)


@permission_required("core.manage_shop", login_url="/login/")
def manage_delivery_time(request, id, template_name="manage/delivery_times/base.html"):
    """Provides a form to edit the delivery time with the passed id.
    """
    delivery_time = get_object_or_404(DeliveryTime, pk=id)
    if request.method == "POST":
        form = DeliveryTimeForm(instance=delivery_time, data=request.POST, files=request.FILES)
        if form.is_valid():
            new_delivery_time = form.save()
            return lfs.core.utils.set_message_cookie(
                url=reverse("lfs_manage_delivery_time", kwargs={"id": id}),
                msg=_(u"Delivery time has been saved."),
            )
    else:
        form = DeliveryTimeForm(instance=delivery_time)

    return render_to_response(template_name, RequestContext(request, {
        "delivery_time": delivery_time,
        "delivery_times": DeliveryTime.objects.all(),
        "form": form,
        "current_id": int(id),
    }))


@permission_required("core.manage_shop", login_url="/login/")
def add_delivery_time(request, template_name="manage/delivery_times/add.html"):
    """Provides a form to add a new delivery time.
    """
    if request.method == "POST":
        form = DeliveryTimeAddForm(data=request.POST)
        if form.is_valid():
            delivery_time = form.save()

            return lfs.core.utils.set_message_cookie(
                url=reverse("lfs_manage_delivery_time", kwargs={"id": delivery_time.id}),
                msg=_(u"Delivery time has been added."),
            )

    else:
        form = DeliveryTimeAddForm()

    return render_to_response(template_name, RequestContext(request, {
        "form": form,
        "delivery_times": DeliveryTime.objects.all(),
        "next": request.REQUEST.get("next", request.META.get("HTTP_REFERER")),
    }))


@require_POST
@permission_required("core.manage_shop", login_url="/login/")
def delete_delivery_time(request, id):
    """Deletes the delivery time with passed id.
    """
    # Remove the delivery time from all products delivery
    for product in Product.objects.filter(delivery_time=id):
        product.delivery_time = None
        product.save()

    # Remove the delivery time from all products order_time
    for product in Product.objects.filter(order_time=id):
        product.order_time = None
        product.save()

    delivery_time = get_object_or_404(DeliveryTime, pk=id)
    delivery_time.delete()

    return lfs.core.utils.set_message_cookie(
        url=reverse("lfs_manage_delivery_times"),
        msg=_(u"Delivery time has been deleted."),
    )



from django import forms
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

class LFSImageInput(forms.FileInput):
    """A custom image widget which displays the current image.
    """
    def render(self, name, value, attrs=None):

        output = super(LFSImageInput, self).render(name, None, attrs=attrs)

        if value and hasattr(value, "url_60x60"):
            output += u"""<div><img src="%s" /></div>""" % value.url_60x60
        elif value and hasattr(value, "url"):
                output += u"""<div><img src="%s" /></div>""" % value.url

        if value:
            trans = _(u"Delete image")
            output += """<div><input type="checkbox" name="delete_image" id="id_delete_image" /> <label for="delete_image">%s</label></div>""" % trans._proxy____unicode_cast()

        return mark_safe(output)



# python imports
import os
import sys

# django imports
from django.core.files.base import ContentFile
from django.http import HttpResponseRedirect


def generate_shipping(request):
    """
    """
    ShippingMethod.objects.all().delete()
    sm1 = ShippingMethod.objects.create(name="Standard", active=True)
    sm2 = ShippingMethod.objects.create(name="Express", active=True)

    pc = CartPriceCriterion.objects.create()
    sm1.criteria_objects.create(criterion=pc)

    return HttpResponseRedirect("/shops")


def generate_categories(request):
    """
    """
    Category.objects.all().delete()

    for i in range(0, 10):
        c = Category(name="Category %s" % i, slug="category-%s" % i)
        c.save()
        for j in range(0, 3):
            sc = Category(name="Category %s%s" % (i, j), slug="category-%s-%s" % (i, j), parent=c)
            sc.save()
            for k in range(0, 3):
                ssc = Category(name="Category %s%s%s" % (i, j, k), slug="category-%s-%s-%s" % (i, j, k), parent=sc)
                ssc.save()

    return HttpResponseRedirect("/shops")


def products(amount=20):
    """
    """
    import lfs.core.utils
    from lfs.catalog.models import Category
    from lfs.catalog.models import Image
    from lfs.catalog.models import Product
    from lfs.core.models import Shop

    from lfs.catalog.models import Property
    from lfs.catalog.models import PropertyOption

    from lfs.shipping.models import ShippingMethod
    from lfs.criteria.models import CartPriceCriterion

    Image.objects.all().delete()
    Product.objects.all().delete()
    Category.objects.all().delete()
    PropertyOption.objects.all().delete()
    Property.objects.all().delete()

    # Images
    path = os.path.join(os.getcwd(), "parts/lfs/lfs/utils/data")
    fh = open(os.path.join(path, "image1.jpg"))
    cf_1 = ContentFile(fh.read())
    fh = open(os.path.join(path, "image2.jpg"))
    cf_2 = ContentFile(fh.read())
    fh = open(os.path.join(path, "image3.jpg"))
    cf_3 = ContentFile(fh.read())

    image_1 = Image(title="Image 1")
    image_1.image.save("Laminat01.jpg", cf_1)
    image_1.save()

    image_2 = Image(title="Image 2")
    image_2.image.save("Laminat02.jpg", cf_2)
    image_2.save()

    image_3 = Image(title="Image 3")
    image_3.image.save("Laminat03.jpg", cf_3)
    image_3.save()

    # Properties
    property = Property(name="Color")
    property.save()

    property_option = PropertyOption(name="Yellow", property=property, price=1.0)
    property_option.save()

    property_option = PropertyOption(name="Red", property=property, price=2.0)
    property_option.save()

    property = Property(name="Size")
    property.save()

    property_option = PropertyOption(name="L", property=property, price=11.0)
    property_option.save()

    property_option = PropertyOption(name="M", property=property, price=12.0)
    property_option.save()

    shop = lfs.core.utils.get_default_shop()

    # Create categories
    category_1 = Category(name="Clothes", slug="clothes")
    category_1.save()

    category_2 = Category(name="Women", slug="women", parent=category_1)
    category_2.save()

    category_3 = Category(name="Pants", slug="pants-woman", parent=category_2)
    category_3.save()

    category_4 = Category(name="Dresses", slug="dresses", parent=category_2)
    category_4.save()

    category_5 = Category(name="Men", slug="men", parent=category_1)
    category_5.save()

    category_6 = Category(name="Pants", slug="pants-men", parent=category_5)
    category_6.save()

    category_7 = Category(name="Pullover", slug="pullover", parent=category_5)
    category_7.save()

    shop.categories = [category_1, category_2, category_3, category_4, category_5, category_6, category_7]
    shop.save()

    # Create products
    for i in range(1, amount):
        p = Product(name="Rock-%s" % i, slug="rock-%s" % i, sku="rock-000%s" % i, price=i * 10)
        p.save()

        if i == 1:
            p.images.add(image_1)
            p.images.add(image_2)
            p.images.add(image_3)
            p.save()
        else:
            img = Image(title="Image 1", image="images/Laminat01.jpg")
            img.save()
            p.images.add(img)
            p.save()

        category_3.products.add(p)
        category_3.save()

        print "Rock-%s created" % i

    for i in range(1, amount):
        p = Product(name="Hemd-%s" % i, slug="hemd-%s" % i, sku="hemd-000%s" % i, price=i * 10)
        p.save()

        img = Image(title="Image 1", image="images/Laminat02.jpg")
        img.save()
        p.images.add(img)
        p.save()

        category_4.products.add(p)
        category_4.save()

        print "Hemd-%s created" % i

    for i in range(1, amount):
        p = Product(name="Pullover-%s" % i, slug="pullover-%s" % i, sku="pullover-000%s" % i, price=i * 10)
        p.save()

        img = Image(title="Image 1", image="images/Laminat03.jpg")
        img.save()
        p.images.add(img)
        p.save()

        category_6.products.add(p)
        category_6.save()

        print "Pullover-%s created" % i

    for i in range(1, amount):
        p = Product(name="Hose-%s" % i, slug="hose-%s" % i, sku="hose-000%s" % i, price=i * 10)
        p.save()

        img = Image(title="Image 1", image="images/Laminat03.jpg")
        img.save()
        p.images.add(img)
        p.save()

        category_7.products.add(p)
        category_7.save()

        print "Hose-%s created" % i



# django imports
from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.models import User


class EmailBackend(ModelBackend):
    """Authenticate against email addresses.
    """
    def authenticate(self, username=None, password=None):
        try:
            user = User.objects.get(email=username)
        except (User.MultipleObjectsReturned, User.DoesNotExist):
            return None
        else:
            if user.check_password(password):
                return user



# django imports
from django.contrib import admin

# lfs imports
from lfs.criteria.models.criteria_objects import CriteriaObjects
from lfs.criteria.models.criteria import CartPriceCriterion
from lfs.criteria.models.criteria import WeightCriterion
from lfs.criteria.models.criteria import CountryCriterion


class CartPriceCriterionAdmin(admin.ModelAdmin):
    """
    """
admin.site.register(CartPriceCriterion, CartPriceCriterionAdmin)


class CountryCriterionAdmin(admin.ModelAdmin):
    """
    """
admin.site.register(CountryCriterion, CountryCriterionAdmin)


class WeightCriterionAdmin(admin.ModelAdmin):
    """
    """
admin.site.register(WeightCriterion, WeightCriterionAdmin)


class CriteriaObjectsAdmin(admin.ModelAdmin):
    """
    """
admin.site.register(CriteriaObjects, CriteriaObjectsAdmin)



# python imports
import re

# django imports
from django.conf import settings
from django.core.cache import cache
from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import ugettext_lazy as _

# lfs imports
import lfs.catalog.utils
from lfs.catalog.models import Product
from lfs.catalog.models import Property
from lfs.catalog.models import PropertyOption

# Load logger
import logging
logger = logging.getLogger("default")


class Cart(models.Model):
    """
    A cart is a container for products which are supposed to be bought by a
    shop customer.

    **Attributes**

    user
       The user to which the cart belongs to

    session
       The session to which the cart belongs to

    creation_date
        The creation date of the cart

    modification_date
        The modification date of the cart

    A cart can be assigned either to the current logged in User (in case
    the shop user is logged in) or to the current session (in case the shop
    user is not logged in).

    A cart is only created if it needs to, i.e. when the shop user adds
    something to the cart.
    """
    user = models.ForeignKey(User, verbose_name=_(u"User"), blank=True, null=True)
    session = models.CharField(_(u"Session"), blank=True, max_length=100)
    creation_date = models.DateTimeField(_(u"Creation date"), auto_now_add=True)
    modification_date = models.DateTimeField(_(u"Modification date"), auto_now=True, auto_now_add=True)

    def __unicode__(self):
        return "%s, %s" % (self.user, self.session)

    # DDD
    @property
    def amount_of_items(self):
        """
        Returns the amount of items of the cart.

        This method is DEPRECATED.
        """
        logger.info("Decprecated: lfs.cart.models.Cart: the property 'amount_of_items' is deprecated. Please use 'get_amount_of_items'.")
        return self.get_amount_of_items()

    # DDD
    def items(self):
        """
        Returns the items of the cart.

        This method is DEPRECATED.
        """
        logger.info("Decprecated: lfs.cart.models.Cart: the method 'items' is deprecated. Please use 'get_items'.")
        return self.get_items()

    # DDD
    def get_name(self):
        """
        Returns a name for the cart.

        This method is DEPRECATED.
        """
        logger.info("Decprecated: lfs.cart.models.Cart: the method 'get_name' is deprecated. (There is no replacement as this method makes no sense at all.)")
        cart_name = ""
        for cart_item in self.items.all():
            if cart_name.product is not None:
                cart_name = cart_name + cart_name.product.get_name() + ", "

        cart_name.strip(', ')
        return cart_name

    def add(self, product, properties=None, amount=1):
        """
        Adds passed product to the cart.

        **Parameters**

        product
            The product which is added.

        properties
            The properties which have been selected by the shop customer
            for the product.

        Returns the newly created cart item.
        """
        if product.is_configurable_product():
            cart_item = self.get_item(product, properties)
            if cart_item:
                cart_item.amount += amount
                cart_item.save()
            else:
                cart_item = CartItem.objects.create(cart=self, product=product, amount=amount)
                if properties:
                    for property_id, value in properties.items():
                        try:
                            Property.objects.get(pk=property_id)
                        except Property.DoesNotExist:
                            pass
                        else:
                            CartItemPropertyValue.objects.create(cart_item=cart_item, property_id=property_id, value=value)
        else:
            try:
                cart_item = CartItem.objects.get(cart=self, product=product)
            except CartItem.DoesNotExist:
                cart_item = CartItem.objects.create(cart=self, product=product, amount=amount)
            else:
                cart_item.amount += amount
                cart_item.save()

        return cart_item

    def get_amount_of_items(self):
        """
        Returns the amount of items of the cart.
        """
        amount = 0
        for item in self.get_items():
            amount += item.amount
        return amount

    def get_item(self, product, properties):
        """
        Returns the item for passed product and properties or None if there
        is none.
        """
        for item in CartItem.objects.filter(cart=self, product=product):
            item_props = {}
            for pv in item.properties.all():
                item_props[unicode(pv.property.id)] = pv.value

            if item_props == properties:
                return item

        return None

    def get_items(self):
        """
        Returns the items of the cart.
        """
        cache_key = "%s-cart-items-%s" % (settings.CACHE_MIDDLEWARE_KEY_PREFIX, self.id)
        items = cache.get(cache_key)
        if items is None:
            items = CartItem.objects.filter(cart=self)
            cache.set(cache_key, items)
        return items

    def get_delivery_time(self, request):
        """
        Returns the delivery time object with the maximal delivery time of all
        products within the cart. Takes the selected shipping method into account.
        """
        import lfs.shipping.utils
        max_delivery_time = None
        for item in self.get_items():
            delivery_time = lfs.shipping.utils.get_product_delivery_time(request, item.product.slug, for_cart=True)
            if (max_delivery_time is None) or (delivery_time.as_hours() > max_delivery_time.as_hours()):
                max_delivery_time = delivery_time
        return max_delivery_time

    def get_price_gross(self, request, total=False):
        """Returns the total gross price of all items.
        """
        price = 0
        for item in self.get_items():
            price += item.get_price_gross(request)
        return price

    def get_price_net(self, request):
        """Returns the total net price of all items.
        """
        price = 0
        for item in self.get_items():
            price += item.get_price_net(request)

        return price

    def get_tax(self, request):
        """Returns the total tax of all items
        """
        tax = 0
        for item in self.get_items():
            tax += item.get_tax(request)

        return tax


class CartItem(models.Model):
    """
    A cart item belongs to a cart. It stores the product and the amount of the
    product which has been taken into the cart.

    **Attributes**

    cart
        The cart the cart item belongs to.

    product
        A reference to a product which is supposed to be bought.

    amount
       Amount of the product which is supposed to be bought.

    creation_date
        The creation date of the cart item.

    modification_date
        The modification date of the cart item.
    """
    cart = models.ForeignKey(Cart, verbose_name=_(u"Cart"))
    product = models.ForeignKey(Product, verbose_name=_(u"Product"))
    amount = models.FloatField(_(u"Quantity"), blank=True, null=True)
    creation_date = models.DateTimeField(_(u"Creation date"), auto_now_add=True)
    modification_date = models.DateTimeField(_(u"Modification date"), auto_now=True, auto_now_add=True)

    class Meta:
        ordering = ['id']

    def __unicode__(self):
        return "Product: %s, Quantity: %f, Cart: %s" % (self.product, self.amount, self.cart)

    # DDD
    def get_price(self, request):
        """
        Convenient method to return the gross price of the product.
        """
        logger.info("Decprecated: lfs.cart.models.CartItem: the method 'get_price' is deprecated. Please use the methods 'get_price_gross' and 'get_price_net'.")
        return self.get_price_gross(request)

    def get_price_net(self, request):
        """
        Returns the total price of the cart item, which is just the multiplication
        of the product's price and the amount of the product within in the cart.
        """
        return (self.get_price_gross(request) * self.amount) - (self.get_tax(request) * self.amount)

    def get_price_gross(self, request):
        """
        Returns the gross item price.
        """
        return self.get_product_price_gross(request) * self.amount

    def get_product_price_gross(self, request):
        """
        Returns the product item price. Based on selected properties, etc.
        """
        if not self.product.is_configurable_product():
            price = self.product.get_price_gross(request)
        else:
            if self.product.active_price_calculation:
                try:
                    price = self.get_calculated_price(request)
                except:
                    price = self.product.get_price_gross(request)
            else:
                price = self.product.get_price_gross(request, with_properties=False)
                for property in self.properties.all():
                    if property.property.is_select_field:
                        try:
                            option = PropertyOption.objects.get(pk=int(float(property.value)))
                        except (PropertyOption.DoesNotExist, AttributeError, ValueError):
                            pass
                        else:
                            price += option.price
        return price

    def get_calculated_price(self, request):
        """
        Returns the calculated gross price of the product based on property
        values and the price calculation field of the product.
        """
        pc = self.product.price_calculation
        tokens = self.product.price_calculation.split(" ")

        for token in tokens:
            if token.startswith("property"):
                mo = re.match("property\((\d+)\)", token)
                ppv = self.properties.filter(property__id=mo.groups()[0])[0]
                if ppv.property.is_select_field:
                    po = PropertyOption.objects.get(pk=ppv.value)
                    value = po.price
                else:
                    value = ppv.value
                pc = pc.replace(token, str(value))
            elif token.startswith("number"):
                mo = re.match("number\((\d+)\)", token)
                pc = pc.replace(token, mo.groups()[0])
            elif token.startswith("product"):
                mo = re.match("product\((.+)\)", token)
                value = getattr(self.product, mo.groups()[0])
                pc = pc.replace(token, str(value))

        return eval(pc)

    def get_tax(self, request):
        """
        Returns the absolute tax of the item.
        """
        rate = self.product.get_tax_rate(request)
        return self.get_price_gross(request) * (rate / (rate + 100))

    def get_properties(self):
        """
        Returns properties of the cart item. Resolves option names for select
        fields.
        """
        properties = []
        for property in self.product.get_properties():
            try:
                cipv = CartItemPropertyValue.objects.get(cart_item=self, property=property)
            except CartItemPropertyValue.DoesNotExist:
                continue

            if property.is_select_field:
                try:
                    option = PropertyOption.objects.get(pk=int(float(cipv.value)))
                except (PropertyOption.DoesNotExist, ValueError):
                    value = cipv.value
                    price = 0.0
                else:
                    value = option.name
                    price = option.price

            else:
                format_string = "%%.%sf" % property.decimal_places
                try:
                    value = format_string % float(cipv.value)
                except ValueError:
                    value = "%.2f" % float(cipv.value)

                price = ""

            properties.append({
                "name": property.name,
                "title": property.title,
                "unit": property.unit,
                "display_price": property.display_price,
                "value": value,
                "price": price,
            })

        return properties


class CartItemPropertyValue(models.Model):
    """
    Stores a value for a property and item.

    **Attributes**

    cart_item
        The cart item - and in this way the product - for which the value
        should be stored.

    property
        The property for which the value should be stored.

    value
        The value which is stored.
    """
    cart_item = models.ForeignKey(CartItem, verbose_name=_(u"Cart item"), related_name="properties")
    property = models.ForeignKey(Property, verbose_name=_(u"Property"))
    value = models.CharField("Value", blank=True, max_length=100)



# django imports
from django.core.urlresolvers import reverse
from django.test import TestCase

# test imports
from lfs.catalog.models import Product


class SearchTestCase(TestCase):
    """Unit tests for lfs.search
    """
    fixtures = ['lfs_shop.xml']

    def setUp(self):
        """
        """
        self.p1 = Product.objects.create(name="Product 1", slug="p1", price=9, active=True)
        self.p2 = Product.objects.create(name="Product 2", slug="p2", price=11, active=True)
        self.p3 = Product.objects.create(name="Product 3", slug="p3", price=13, active=False)

    def test_search(self):
        """
        """
        url = reverse("lfs_search")

        # Must be found
        response = self.client.get(url, {"q": "Product"})
        self.failIf(response.content.find("Product 1") == -1)

        # Must not be found
        response = self.client.get(url, {"q": "Hurz"})
        self.failIf(response.content.find("Product 1") != -1)

    def test_inactive_search(self):
        """
        """
        url = reverse("lfs_search")

        # Must not be found
        response = self.client.get(url, {"q": "Product"})
        self.failIf(response.content.find("Product 3") != -1)



# python imports
from datetime import datetime

# django imports
from django.conf import settings
from django.core.cache import cache
from django.core.exceptions import ObjectDoesNotExist

# lfs imports
import lfs.core.utils
from lfs.caching.utils import lfs_get_object_or_404
from lfs.catalog.models import DeliveryTime
from lfs.catalog.models import Product
from lfs.catalog.settings import DELIVERY_TIME_UNIT_DAYS
from lfs.catalog.settings import PRODUCT_WITH_VARIANTS
from lfs.criteria import utils as criteria_utils
from lfs.customer import utils as customer_utils
from lfs.shipping.models import ShippingMethod


def get_product_delivery_time(request, product_slug, for_cart=False):
    """Returns the delivery time object for the product with given slug.

    If the ``for_cart`` parameter is False, the default delivery time for
    product is calculated. This is at the moment the first valid (iow with the
    hightest priority) shipping method.

    If the ``for_cart parameter`` is True, the delivery time for the product
    within the cart is calculated. This can differ because the shop customer has
    the opportunity to select a shipping method within the cart. If this
    shipping method is valid for the given product this one is taken, if not
    the default one - the default one is the first valid shipping method.
    """
    # TODO: Need a reasonable chaching here
    if for_cart:
        cache_key = "%s-shipping-delivery-time-cart-%s" % (settings.CACHE_MIDDLEWARE_KEY_PREFIX, request.user.id)
    else:
        cache_key = "%s-shipping-delivery-time" % settings.CACHE_MIDDLEWARE_KEY_PREFIX

    shippings = None  # cache.get(cache_key)
    if shippings is None:
        shippings = {}

    product_key = "product-%s" % product_slug
    shipping = shippings.get(product_key)
    if shipping is not None:
        return shipping

    product = lfs_get_object_or_404(Product, slug=product_slug)

    # if the product is a product with variants we switch to the default
    # variant to calculate the delivery time. Please note that in this case
    # the default variant is also displayed.
    if product.sub_type == PRODUCT_WITH_VARIANTS:
        variant = product.get_default_variant()
        if variant is not None:
            product = variant

    if product.manual_delivery_time:
        delivery_time = product.delivery_time
    else:
        if for_cart:
            sm = get_selected_shipping_method(request)
            # Within the cart we have to take care of the selected shipping
            # method.
            sms = get_valid_shipping_methods(request, product)
            if sm in sms:
                try:
                    delivery_time = sm.delivery_time
                except AttributeError:
                    delivery_time = None
            else:
                sm = get_default_shipping_method(request)
                try:
                    delivery_time = sm.delivery_time
                except AttributeError:
                    delivery_time = None
        else:
            # For the product we take the standard shipping method, which is the
            # first valid shipping method at the moment.
            try:
                shipping_method = get_first_valid_shipping_method(request, product)
                delivery_time = shipping_method.delivery_time
            except AttributeError:
                delivery_time = None

    # TODO: Define default delivery time for the shop
    if delivery_time is None:
        delivery_time = DeliveryTime(min=1, max=2, unit=DELIVERY_TIME_UNIT_DAYS)

    # Calculate the total delivery time if the product is not on stock.
    if (product.stock_amount <= 0) and (product.order_time):

        # Calculate how much days are left until the product is going to be
        # delivered.
        if product.ordered_at:
            order_delta = datetime.now().date() - product.ordered_at
            order_delta = order_delta.days
        else:
            order_delta = 0

        # Calculate the rest of the origin order time.
        order_time_left = product.order_time.subtract_days(order_delta)

        # Calculate the total delivery time.
        delivery_time += order_time_left
        delivery_time = delivery_time.as_reasonable_unit()

    delivery_time = delivery_time.round()
    shippings[product_key] = delivery_time
    cache.set(cache_key, shippings)

    return delivery_time


def update_to_valid_shipping_method(request, customer, save=False):
    """After this has been called the given customer has a valid shipping
    method in any case.
    """
    valid_sms = get_valid_shipping_methods(request)

    if customer.selected_shipping_method not in valid_sms:
        customer.selected_shipping_method = get_default_shipping_method(request)
        if save:
            customer.save()


def get_valid_shipping_methods(request, product=None):
    """Returns a list of all valid shipping methods for the passed request.
    """
    result = []
    for sm in ShippingMethod.objects.filter(active=True):
        if criteria_utils.is_valid(request, sm, product):
            result.append(sm)
    return result


def get_first_valid_shipping_method(request, product=None):
    """Returns the valid shipping method with the highest priority.
    """
    active_shipping_methods = ShippingMethod.objects.filter(active=True)
    return criteria_utils.get_first_valid(request, active_shipping_methods, product)


def get_default_shipping_method(request):
    """Returns the default shipping method for the passed request.

    At the moment is this the first valid shipping method, but this could be
    made more explicit in future.
    """
    active_shipping_methods = ShippingMethod.objects.filter(active=True)
    return criteria_utils.get_first_valid(request, active_shipping_methods)


def get_selected_shipping_method(request):
    """Returns the selected shipping method for the passed request.

    This could either be an explicitely selected shipping method of the current
    user or the default shipping method.
    """
    customer = customer_utils.get_customer(request)
    if customer and customer.selected_shipping_method:
        return customer.selected_shipping_method
    else:
        return get_default_shipping_method(request)


def get_selected_shipping_country(request):
    """Returns the selected shipping country for the passed request.

    This could either be an explicitely selected country of the current
    user or the default country of the shop.
    """
    customer = customer_utils.get_customer(request)
    if customer:
        if customer.selected_shipping_address:
            return customer.selected_shipping_address.country
        elif customer.selected_country:
            return customer.selected_country

    return lfs.core.utils.get_default_shop().get_default_country()


def get_shipping_costs(request, shipping_method):
    """Returns a dictionary with the shipping price and tax for the passed
    request and shipping method.

    The format of the dictionary is: {"price" : 0.0, "tax" : 0.0}
    """
    if shipping_method is None:
        return {
            "price": 0.0,
            "tax": 0.0
        }

    try:
        tax_rate = shipping_method.tax.rate
    except AttributeError:
        tax_rate = 0.0

    price = criteria_utils.get_first_valid(request,
        shipping_method.prices.all())

    if price is None:
        price = shipping_method.price
        tax = (tax_rate / (tax_rate + 100)) * price

        return {
            "price": price,
            "tax": tax
        }
    else:
        tax = (tax_rate / (tax_rate + 100)) * price.price

        return {
            "price": price.price,
            "tax": tax
        }


def get_delivery_time(request, product):
    """Returns delivery time for given product.
    """
    if product.deliverable == False:
        return {
            "deliverable": False,
            "delivery_time": get_product_delivery_time(request, product.slug)
        }
    else:
        return {
            "deliverable": True,
            "delivery_time": get_product_delivery_time(request, product.slug)
        }



# -*- coding: utf-8 -*-

# python imports
import re

# django imports
from django.http import HttpResponse

# tagging imports
from tagging.models import Tag

# lfs imports
from lfs.catalog.models import Product
from lfs.tagging import utils as tagging_utils
from lfs.tagging.settings import RE_STOP_WORDS
from lfs.tagging.settings import RE_SEPARATORS


def tag_products(request, source="description"):
    """Auto tags product on base of product description.
    """
    if source == "description":
        parser = tagging_utils.SimpleHTMLParser()
        for product in Product.objects.all():
            print product.name
            parser.feed(product.description)
            Tag.objects.update_tags(product, "")

            data, amount = re.subn(r"[\W]*", "", parser.data)
            tags = re.split("\s*", data)
            for tag in tags:
                if tag:
                    Tag.objects.add_tag(product, tag)

    elif source == "name":
        for product in Product.objects.all():
            Tag.objects.update_tags(product, "")

            data, amount = RE_STOP_WORDS.subn("", product.name)
            data, amount = RE_SEPARATORS.subn(" ", data)

            tags = re.split("\s*", data)

            for tag in tags:
                if tag:
                    Tag.objects.add_tag(product, tag)

        return HttpResponse("")



# django imports
from django.contrib.auth.decorators import permission_required
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.forms import ModelForm
from django.http import HttpResponseRedirect
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.shortcuts import render_to_response
from django.template.loader import render_to_string
from django.template import RequestContext
from django.utils import simplejson
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.http import require_POST

# lfs imports
import lfs.core.utils
from lfs.caching.utils import lfs_get_object_or_404
from lfs.core.utils import LazyEncoder
from lfs.core.widgets.image import LFSImageInput
from lfs.criteria import utils as criteria_utils
from lfs.customer.models import Customer
from lfs.discounts.models import Discount


class DiscountForm(ModelForm):
    """Form to manage discount data.
    """
    class Meta:
        model = Discount


@permission_required("core.manage_shop", login_url="/login/")
def manage_discounts(request):
    """Dispatches to the first discount or to the add discount method
    form if there is no discount yet.
    """
    try:
        discount = Discount.objects.all()[0]
    except IndexError:
        url = reverse("lfs_manage_add_discount")
    else:
        url = reverse("lfs_manage_discount", kwargs={"id": discount.id})

    return HttpResponseRedirect(url)


@permission_required("core.manage_shop", login_url="/login/")
def manage_discount(request, id, template_name="manage/discounts/discount.html"):
    """The main view to manage the discount with given id.

    This view collects the various parts of the discount form (data, criteria,
    and displays them.
    """
    discount = Discount.objects.get(pk=id)

    return render_to_response(template_name, RequestContext(request, {
        "discount": discount,
        "navigation": navigation(request),
        "data": discount_data(request, id),
        "criteria": discount_criteria(request, id),
    }))


# Parts of the manage discount view.
@permission_required("core.manage_shop", login_url="/login/")
def navigation(request, template_name="manage/discounts/navigation.html"):
    """Returns the navigation for the discount view.
    """
    try:
        current_id = int(request.path.split("/")[-1])
    except ValueError:
        current_id = ""

    return render_to_string(template_name, RequestContext(request, {
        "current_id": current_id,
        "discounts": Discount.objects.all(),
    }))


@permission_required("core.manage_shop", login_url="/login/")
def discount_data(request, id, template_name="manage/discounts/data.html"):
    """Returns the discount data as html.

    This view is used as a part within the manage discount view.
    """
    discount = Discount.objects.get(pk=id)

    return render_to_string(template_name, RequestContext(request, {
        "form": DiscountForm(instance=discount),
        "discount": discount,
    }))


@permission_required("core.manage_shop", login_url="/login/")
def discount_criteria(request, id, template_name="manage/discounts/criteria.html"):
    """Returns the criteria of the discount with passed id as HTML.

    This view is used as a part within the manage discount view.
    """
    discount = Discount.objects.get(pk=id)

    criteria = []
    position = 0
    for criterion_object in discount.criteria_objects.all():
        position += 10
        criterion_html = criterion_object.criterion.as_html(request, position)
        criteria.append(criterion_html)

    return render_to_string(template_name, RequestContext(request, {
        "discount": discount,
        "criteria": criteria,
    }))


# Actions
@permission_required("core.manage_shop", login_url="/login/")
def add_discount(request, template_name="manage/discounts/add_discount.html"):
    """Provides an add form and saves a new discount method.
    """
    if request.method == "POST":
        form = DiscountForm(data=request.POST, files=request.FILES)
        if form.is_valid():
            new_discount = form.save()
            return lfs.core.utils.set_message_cookie(
                url=reverse("lfs_manage_discount", kwargs={"id": new_discount.id}),
                msg=_(u"Discount method has been added."),
            )
    else:
        form = DiscountForm()

    return render_to_response(template_name, RequestContext(request, {
        "navigation": navigation(request),
        "form": form,
    }))


@permission_required("core.manage_shop", login_url="/login/")
def save_discount_criteria(request, id):
    """Saves the criteria for the discount with given id. The criteria
    are passed via request body.
    """
    discount = lfs_get_object_or_404(Discount, pk=id)
    criteria_utils.save_criteria(request, discount)
    criteria = discount_criteria(request, id)

    html = [["#criteria", discount_criteria(request, id)]]

    result = simplejson.dumps({
        "html": html,
        "message": _("Modifications have been changed."),
    }, cls=LazyEncoder)

    return HttpResponse(result)


@permission_required("core.manage_shop", login_url="/login/")
def save_discount_data(request, id):
    """Saves discount data (via request body) to the discount with passed
    id.

    This is called via an AJAX request and returns JSON encoded data.
    """
    discount = Discount.objects.get(pk=id)
    discount_form = DiscountForm(instance=discount, data=request.POST)

    form = render_to_string(
        "manage/discounts/data.html", RequestContext(request, {
        "form": discount_form,
        "discount": discount,
    }))

    if discount_form.is_valid():
        discount_form.save()

    return lfs.core.utils.set_message_cookie(
        url=reverse("lfs_manage_discount", kwargs={"id": id}),
        msg=_(u"Discount data has been saved."),
    )


@require_POST
@permission_required("core.manage_shop", login_url="/login/")
def delete_discount(request, id):
    """Deletes discount with passed id.
    """
    try:
        discount = Discount.objects.get(pk=id)
    except ObjectDoesNotExist:
        pass
    else:
        discount.delete()

    return lfs.core.utils.set_message_cookie(
        url=reverse("lfs_manage_discounts"),
        msg=_(u"Discount has been deleted."),
    )



# python imports
import os
import csv

# django imports
from django.core.exceptions import ObjectDoesNotExist
from django.core.files.base import ContentFile
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template.defaultfilters import slugify

# django imports
from django.http import HttpResponse

# lfs imports
from lfs.catalog.models import Category
from lfs.catalog.models import Product
from lfs.catalog.models import ProductAccessories
from lfs.catalog.models import Image


def test(request):
    """
    """
    return render_to_response("test.html", RequestContext(request))


def upload_test(request):
    """
    """
    if request.method == "GET":
        return render_to_response("testuploadform.html")

    return HttpResponse()



# django imports
from django.contrib.auth.decorators import permission_required
from django.core.urlresolvers import reverse
from django.forms import ModelForm
from django.http import HttpResponse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils import simplejson
from django.utils.translation import ugettext_lazy as _

# lfs imports
import lfs.core.utils
from lfs.caching.utils import lfs_get_object_or_404
from lfs.core.models import Shop
from lfs.core.signals import shop_changed
from lfs.core.utils import LazyEncoder
from lfs.core.widgets.image import LFSImageInput
from lfs.manage.views.lfs_portlets import portlets_inline


class ShopForm(ModelForm):
    """Form to edit shop data.
    """
    def __init__(self, *args, **kwargs):
        super(ShopForm, self).__init__(*args, **kwargs)
        self.fields["image"].widget = LFSImageInput()

    class Meta:
        model = Shop
        fields = ("name", "shop_owner", "from_email", "notification_emails",
            "description", "image", "static_block", "checkout_type", "confirm_toc",
            "google_analytics_id", "ga_site_tracking", "ga_ecommerce_tracking")


class ShopDefaultValuesForm(ModelForm):
    """
    """
    class Meta:
        model = Shop
        fields = ("price_calculator", "product_cols", "product_rows", "category_cols",
            "default_country", "invoice_countries", "shipping_countries", "default_locale", "use_international_currency_code")


@permission_required("core.manage_shop", login_url="/login/")
def manage_shop(request, template_name="manage/shop/shop.html"):
    """Displays the form to manage shop data.
    """
    shop = lfs_get_object_or_404(Shop, pk=1)
    if request.method == "POST":
        form = ShopForm(instance=shop, data=request.POST, files=request.FILES)
        if form.is_valid():
            form.save()
            return lfs.core.utils.set_message_cookie(
                url=reverse("lfs_manage_shop"),
                msg=_(u"Shop data has been saved."),
            )
    else:
        form = ShopForm(instance=shop)

    return render_to_response(template_name, RequestContext(request, {
        "shop": shop,
        "form": form,
        "default_values": default_values_part(request),
        "portlets": portlets_inline(request, shop),
    }))


@permission_required("core.manage_shop", login_url="/login/")
def default_values_part(request, template_name="manage/shop/default_values.html"):
    """Displays the default value part of the shop form.
    """
    shop = lfs_get_object_or_404(Shop, pk=1)

    if request.method == "POST":
        form = ShopDefaultValuesForm(instance=shop, data=request.POST)
    else:
        form = ShopDefaultValuesForm(instance=shop)

    return render_to_string(template_name, RequestContext(request, {
        "shop": shop,
        "form": form,
    }))


def save_default_values(request):
    """Saves the default value part
    """
    shop = lfs_get_object_or_404(Shop, pk=1)
    form = ShopDefaultValuesForm(instance=shop, data=request.POST)

    if form.is_valid():
        shop = form.save()
        shop_changed.send(shop)
        message = _(u"Shop default values have been saved.")
    else:
        message = _(u"Please correct the indicated errors.")

    html = [["#default-values", default_values_part(request)]]

    result = simplejson.dumps({
        "html": html,
        "message": message
    }, cls=LazyEncoder)

    return HttpResponse(result)



# django imports
from django.contrib.auth.decorators import permission_required
from django.core.urlresolvers import reverse
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _
from django import forms

# lfs imports
import lfs.caching.utils
import lfs.core.utils
import lfs.catalog.models
import lfs.marketing.utils


@permission_required("core.manage_shop", login_url="/login/")
def utilities(request, template_name="manage/utils.html"):
    """Displays the utility view.
    """
    return render_to_response(template_name, RequestContext(request, {}))


@permission_required("core.manage_shop", login_url="/login/")
def reindex_topseller(request):
    """Clears and reindexes the topsellers.
    """
    lfs.marketing.utils.calculate_product_sales()
    return lfs.core.utils.set_message_cookie(
        url=reverse("lfs_manage_utils"),
        msg=_(u"Topseller have been reindexed."),
    )


@permission_required("core.manage_shop", login_url="/login/")
def clear_cache(request):
    """Clears the whole cache.
    """
    lfs.caching.utils.clear_cache()
    return lfs.core.utils.set_message_cookie(
        url=reverse("lfs_manage_utils"),
        msg=_(u"Cache has been cleared."),
    )


@permission_required("core.manage_shop", login_url="/login/")
def set_category_levels(request):
    """Sets the category levels based on the position in category hierarchy.
    """
    lfs.core.utils.set_category_levels()
    return lfs.core.utils.set_message_cookie(
        url=reverse("lfs_manage_utils"),
        msg=_(u"Categoy levels have been created."),
    )


@permission_required("core.manage_shop", login_url="/login/")
def update_effective_price(request):
    """Saves the price or sale price to effective price.
    """
    for product in lfs.catalog.models.Product.objects.all():
        product.save()

    return lfs.core.utils.set_message_cookie(
        url=reverse("lfs_manage_utils"),
        msg=_(u"Effective prices have been set."),
    )



# django imports
from django.contrib.auth.decorators import permission_required
from django.db import IntegrityError
from django.core.urlresolvers import reverse
from django.forms import ModelForm
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils import simplejson
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.http import require_POST

# lfs imports
import lfs.core.utils
from lfs.catalog.models import GroupsPropertiesRelation
from lfs.catalog.models import Property
from lfs.catalog.models import PropertyGroup
from lfs.core.utils import LazyEncoder
from lfs.manage.views import property_groups


class PropertyGroupForm(ModelForm):
    """
    """
    class Meta:
        model = PropertyGroup
        fields = ["name"]


@permission_required("core.manage_shop", login_url="/login/")
def manage_property_groups(request):
    """The main view to manage properties.
    """
    try:
        property = PropertyGroup.objects.all()[0]
        url = reverse("lfs_manage_property_group", kwargs={"id": property.id})
    except IndexError:
        url = reverse("lfs_add_property_group")

    return HttpResponseRedirect(url)


@permission_required("core.manage_shop", login_url="/login/")
def manage_property_group(request, id, template_name="manage/properties/property_group.html"):
    """Edits property group with given id.
    """
    property_group = get_object_or_404(PropertyGroup, pk=id)
    if request.method == "POST":
        form = PropertyGroupForm(instance=property_group, data=request.POST)
        if form.is_valid():
            new_property_group = form.save()
            return lfs.core.utils.set_message_cookie(
                url=reverse("lfs_manage_property_group", kwargs={"id": property_group.id}),
                msg=_(u"Property group has been saved."),
            )
    else:
        form = PropertyGroupForm(instance=property_group)

    return render_to_response(template_name, RequestContext(request, {
        "property_group": property_group,
        "property_groups": PropertyGroup.objects.all(),
        "properties": properties_inline(request, id),
        "products": property_groups.products.products(request, id),
        "form": form,
        "current_id": int(id),
    }))


@permission_required("core.manage_shop", login_url="/login/")
def properties_inline(request, id, template_name="manage/properties/properties_inline.html"):
    """
    """
    property_group = get_object_or_404(PropertyGroup, pk=id)

    gps = GroupsPropertiesRelation.objects.filter(group=id)

    # Calculate assignable properties
    assigned_property_ids = [p.property.id for p in gps]
    assignable_properties = Property.objects.exclude(
        pk__in=assigned_property_ids).exclude(local=True)

    return render_to_string(template_name, RequestContext(request, {
        "property_group": property_group,
        "properties": assignable_properties,
        "gps": gps,
    }))


@permission_required("core.manage_shop", login_url="/login/")
def add_property_group(request, template_name="manage/properties/add_property_group.html"):
    """Adds a new property group
    """
    if request.method == "POST":
        form = PropertyGroupForm(data=request.POST)
        if form.is_valid():
            property_group = form.save()
            return lfs.core.utils.set_message_cookie(
                url=reverse("lfs_manage_property_group", kwargs={"id": property_group.id}),
                msg=_(u"Property group has been added."),
            )
    else:
        form = PropertyGroupForm()

    return render_to_response(template_name, RequestContext(request, {
        "form": form,
        "property_groups": PropertyGroup.objects.all(),
    }))


@require_POST
@permission_required("core.manage_shop", login_url="/login/")
def delete_property_group(request, id):
    """Deletes the property group with passed id.
    """
    property_group = get_object_or_404(PropertyGroup, pk=id)
    property_group.delete()

    return lfs.core.utils.set_message_cookie(
        url=reverse("lfs_manage_property_groups"),
        msg=_(u"Property group has been deleted."),
    )


@permission_required("core.manage_shop", login_url="/login/")
def assign_properties(request, group_id):
    """Assignes given properties (via request body) to passed group id.
    """
    for property_id in request.POST.getlist("property-id"):
        try:
            GroupsPropertiesRelation.objects.create(group_id=group_id, property_id=property_id)
        except IntegrityError:
            pass

    _udpate_positions(group_id)

    html = [["#properties", properties_inline(request, group_id)]]
    result = simplejson.dumps({
        "html": html,
        "message": _(u"Properties have been assigned.")
    }, cls=LazyEncoder)

    return HttpResponse(result)


@permission_required("core.manage_shop", login_url="/login/")
def update_properties(request, group_id):
    """Update or Removes given properties (via request body) from passed group id.
    """
    if request.POST.get("action") == "remove":
        for property_id in request.POST.getlist("property-id"):
            try:
                gp = GroupsPropertiesRelation.objects.get(group=group_id, property=property_id)
            except GroupsPropertiesRelation.DoesNotExist:
                pass
            else:
                gp.delete()

        message = _(u"Properties have been removed.")

    else:
        for gp in GroupsPropertiesRelation.objects.filter(group=group_id):
            position = request.POST.get("position-%s" % gp.property.id, 999)
            gp.position = int(position)
            gp.save()
            message = _(u"Properties have been updated.")

    _udpate_positions(group_id)

    html = [["#properties", properties_inline(request, group_id)]]
    result = simplejson.dumps({
        "html": html,
        "message": message
    }, cls=LazyEncoder)

    return HttpResponse(result)


def _udpate_positions(group_id):
    """
    """
    for i, gp in enumerate(GroupsPropertiesRelation.objects.filter(group=group_id)):
        gp.position = (i + 1) * 10
        gp.save()



from lfs.catalog.models import Product

from djangorestframework.resources import ModelResource


class ProductResource(ModelResource):
    """
    A Rest representation of an LFS Product
    """
    model = Product
    #fields = ['active]



# django imports
from django import forms
from django.template import RequestContext
from django.template.loader import render_to_string

# portlets imports
from portlets.models import Portlet


class RelatedProductsPortlet(Portlet):
    """Portlet to display related products.
    """
    class Meta:
        app_label = 'portlet'

    def __unicode__(self):
        return "%s" % self.id

    def render(self, context):
        """Renders the portlet as html.
        """
        product = context.get("product")
        request = context.get("request")

        return render_to_string("lfs/portlets/related_products.html", RequestContext(request, {
            "title": self.title,
            "product": product,
        }))

    def form(self, **kwargs):
        return RelatedProductsForm(instance=self, **kwargs)


class RelatedProductsForm(forms.ModelForm):
    """Form for the RelatedProductsPortlet.
    """
    class Meta:
        model = RelatedProductsPortlet



# django imports
from django.db import models
from django.utils.translation import ugettext_lazy as _

# lfs imports
from lfs.catalog.models import Product
from lfs.order.models import Order


class Topseller(models.Model):
    """Selected products are in any case among topsellers.
    """
    product = models.ForeignKey(Product, verbose_name=_(u"Product"))
    position = models.PositiveSmallIntegerField(_(u"Position"), default=1)

    class Meta:
        ordering = ["position"]

    def __unicode__(self):
        return "%s (%s)" % (self.product.name, self.position)


class ProductSales(models.Model):
    """Stores totals sales per product.
    """
    product = models.ForeignKey(Product, verbose_name=_(u"Product"))
    sales = models.IntegerField(_(u"sales"), default=0)


class FeaturedProduct(models.Model):
    """Featured products are manually selected by the shop owner
    """
    product = models.ForeignKey(Product, verbose_name=_(u"Product"))
    position = models.PositiveSmallIntegerField(_(u"Position"), default=1)
    active = models.BooleanField(_(u"Active"), default=True)

    class Meta:
        ordering = ["position"]

    def __unicode__(self):
        return "%s (%s)" % (self.product.name, self.position)


class OrderRatingMail(models.Model):
    """Saves whether and when a rating mail has been send for an order.
    """
    order = models.ForeignKey(Order, verbose_name=_(u"Order"))
    send_date = models.DateTimeField(auto_now=True)

    def __unicode__(self):
        return "%s (%s)" % (self.order.id, self.rating_mail_sent)



# django imports
from django.contrib.auth.decorators import permission_required
from django.core.exceptions import ObjectDoesNotExist
from django.core.urlresolvers import reverse
from django.forms import ModelForm
from django.http import HttpResponseRedirect
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.shortcuts import render_to_response
from django.template.loader import render_to_string
from django.template import RequestContext
from django.utils import simplejson
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.http import require_POST

# lfs imports
import lfs.core.utils
from lfs.caching.utils import lfs_get_object_or_404
from lfs.core.utils import LazyEncoder
from lfs.core.widgets.image import LFSImageInput
from lfs.criteria import utils as criteria_utils
from lfs.customer.models import Customer
from lfs.shipping.models import ShippingMethod
from lfs.shipping.models import ShippingMethodPrice
from lfs.shipping import utils as shipping_utils


class ShippingMethodAddForm(ModelForm):
    """Form to add a shipping method.
    """
    class Meta:
        model = ShippingMethod
        fields = ["name"]


class ShippingMethodForm(ModelForm):
    """
    """
    def __init__(self, *args, **kwargs):
        super(ShippingMethodForm, self).__init__(*args, **kwargs)
        self.fields["image"].widget = LFSImageInput()

    class Meta:
        model = ShippingMethod


# # Starting pages. This pages are called directly via a request
@permission_required("core.manage_shop", login_url="/login/")
def manage_shipping(request):
    """Dispatches to the first shipping method or to the add shipping method
    form if there is no shipping method.
    """
    try:
        shipping_method = ShippingMethod.objects.all()[0]
    except IndexError:
        url = reverse("lfs.manage.views.add_shipping_method")
    else:
        url = reverse("lfs.manage.views.manage_shipping_method",
            kwargs={"shipping_method_id": shipping_method.id})
    return HttpResponseRedirect(url)


@permission_required("core.manage_shop", login_url="/login/")
def manage_shipping_method(request, shipping_method_id,
    template_name="manage/shipping/manage_shipping.html"):
    """The main view to manage the shipping method with given id.

    This view collects the various parts of the shipping form (data, criteria,
    prices) and displays them.
    """
    shipping_method = ShippingMethod.objects.get(pk=shipping_method_id)

    return render_to_response(template_name, RequestContext(request, {
        "shipping_method": shipping_method,
        "shipping_methods": shipping_methods(request),
        "data": shipping_method_data(request, shipping_method_id),
        "method_criteria": shipping_method_criteria(request, shipping_method_id),
        "method_prices": shipping_method_prices(request, shipping_method_id),
    }))


# Parts of the manage shipping view.
@permission_required("core.manage_shop", login_url="/login/")
def shipping_methods(request, template_name="manage/shipping/shipping_methods.html"):
    """Returns all shipping methods as html.

    This view is used as a part within the manage shipping view.
    """
    try:
        current_id = int(request.path.split("/")[-1])
    except ValueError:
        current_id = ""

    return render_to_string(template_name, RequestContext(request, {
        "current_id": current_id,
        "shipping_methods": ShippingMethod.objects.all(),
    }))


@permission_required("core.manage_shop", login_url="/login/")
def shipping_method_data(request, shipping_id,
    template_name="manage/shipping/shipping_method_data.html"):
    """Returns the shipping data as html.

    This view is used as a part within the manage shipping view.
    """
    shipping_method = ShippingMethod.objects.get(pk=shipping_id)

    return render_to_string(template_name, RequestContext(request, {
        "form": ShippingMethodForm(instance=shipping_method),
        "shipping_method": shipping_method,
    }))


@permission_required("core.manage_shop", login_url="/login/")
def shipping_method_criteria(request, shipping_method_id,
    template_name="manage/shipping/shipping_method_criteria.html"):
    """Returns the criteria of the shipping method with passed id as HTML.

    This view is used as a part within the manage shipping view.
    """
    shipping_method = ShippingMethod.objects.get(pk=shipping_method_id)

    criteria = []
    position = 0
    for criterion_object in shipping_method.criteria_objects.all():
        position += 10
        criterion_html = criterion_object.criterion.as_html(request, position)
        criteria.append(criterion_html)

    return render_to_string(template_name, RequestContext(request, {
        "shipping_method": shipping_method,
        "criteria": criteria,
    }))


@permission_required("core.manage_shop", login_url="/login/")
def shipping_method_prices(request, shipping_method_id,
    template_name="manage/shipping/shipping_method_prices.html"):
    """Returns the shipping method prices for the shipping method with given id.

    This view is used as a part within the manage shipping view.
    """
    shipping_method = get_object_or_404(ShippingMethod, pk=shipping_method_id)

    return render_to_string(template_name, RequestContext(request, {
        "shipping_method": shipping_method,
        "prices": shipping_method.prices.all(),
    }))


@permission_required("core.manage_shop", login_url="/login/")
def shipping_price_criteria(request, shipping_price_id, as_string=False,
    template_name="manage/shipping/shipping_price_criteria.html"):
    """Returns the criteria of the shipping price with passed id.

    This view is used as a part within the manage shipping view.
    """
    shipping_price = get_object_or_404(ShippingMethodPrice, pk=shipping_price_id)

    criteria = []
    position = 0
    for criterion_object in shipping_price.criteria_objects.all():
        position += 10
        criterion_html = criterion_object.criterion.as_html(request, position)
        criteria.append(criterion_html)

    dialog = render_to_string(template_name, RequestContext(request, {
        "shipping_price": shipping_price,
        "criteria": criteria,
    }))

    if as_string:
        return dialog
    else:
        html = [["#dialog", dialog]]

        result = simplejson.dumps({
            "html": html,
            "open-dialog": True,
        }, cls=LazyEncoder)

        return HttpResponse(result)


@permission_required("core.manage_shop", login_url="/login/")
def add_shipping_method(request,
    template_name="manage/shipping/add_shipping_method.html"):
    """Provides an add form and saves a new shipping method.
    """
    if request.method == "POST":
        form = ShippingMethodAddForm(data=request.POST)
        if form.is_valid():
            new_shipping_method = form.save()
            return lfs.core.utils.set_message_cookie(
                url=reverse("lfs.manage.views.manage_shipping_method", kwargs={"shipping_method_id": new_shipping_method.id}),
                msg=_(u"Shipping method has been added."),
            )
    else:
        form = ShippingMethodAddForm()

    return render_to_response(template_name, RequestContext(request, {
        "form": form,
        "next": request.REQUEST.get("next", request.META.get("HTTP_REFERER")),
    }))


# Actions
@permission_required("core.manage_shop", login_url="/login/")
def save_shipping_method_criteria(request, shipping_method_id):
    """Saves the criteria for the shipping method with given id. The criteria
    are passed via request body.
    """
    shipping_method = lfs_get_object_or_404(ShippingMethod, pk=shipping_method_id)

    criteria_utils.save_criteria(request, shipping_method)

    html = [["#criteria", shipping_method_criteria(request, shipping_method_id)]]

    result = simplejson.dumps({
        "html": html,
        "message": _(u"Modifications have been changed."),
    }, cls=LazyEncoder)

    return HttpResponse(result)


@permission_required("core.manage_shop", login_url="/login/")
def save_shipping_price_criteria(request, shipping_price_id):
    """Saves the criteria for the shipping price with given id. The criteria
    are passed via request body.
    """
    shipping_price = get_object_or_404(ShippingMethodPrice, pk=shipping_price_id)

    criteria_utils.save_criteria(request, shipping_price)

    html = [
        ["#price-criteria", shipping_price_criteria(request, shipping_price_id, as_string=True)],
        ["#prices", shipping_method_prices(request, shipping_price.shipping_method.id)],
    ]

    result = simplejson.dumps({
        "html": html,
        "message": _(u"Modifications have been changed."),
    }, cls=LazyEncoder)

    return HttpResponse(result)


@permission_required("core.manage_shop", login_url="/login/")
def add_shipping_price(request, shipping_method_id):
    """Adds given shipping price (via request body) to shipping method with
    give id.

    Returns JSON encoded data.
    """
    try:
        price = float(request.POST.get("price", 0))
    except ValueError:
        price = 0.0

    shipping_method = get_object_or_404(ShippingMethod, pk=shipping_method_id)
    shipping_method.prices.create(price=price)
    _update_price_positions(shipping_method)

    message = _(u"Price has been added")
    html = [["#prices", shipping_method_prices(request, shipping_method_id)]]

    result = simplejson.dumps({
        "html": html,
        "message": message,
    }, cls=LazyEncoder)

    return HttpResponse(result)


@permission_required("core.manage_shop", login_url="/login/")
def update_shipping_prices(request, shipping_method_id):
    """Saves/Deletes shipping prices with passed ids (via request body)
    dependent on given action (via request body).
    """
    shipping_method = get_object_or_404(ShippingMethod, pk=shipping_method_id)

    action = request.POST.get("action")
    if action == "delete":
        message = _(u"Prices have been deleted")
        for key in request.POST.keys():
            if key.startswith("delete-"):
                try:
                    id = key.split("-")[1]
                    price = get_object_or_404(ShippingMethodPrice, pk=id)
                except (IndexError, ObjectDoesNotExist):
                    continue
                else:
                    price.delete()

    elif action == "update":
        message = _(u"Prices have been updated")
        for key, value in request.POST.items():
            if key.startswith("price-"):
                try:
                    id = key.split("-")[1]
                    price = get_object_or_404(ShippingMethodPrice, pk=id)
                except (IndexError, ObjectDoesNotExist):
                    continue
                else:
                    try:
                        value = float(value)
                    except ValueError:
                        value = 0.0
                    price.price = value
                    price.priority = request.POST.get("priority-%s" % id, 0)
                    price.save()

    _update_price_positions(shipping_method)

    html = [["#prices", shipping_method_prices(request, shipping_method_id)]]
    result = simplejson.dumps({
        "html": html,
        "message": message,
    }, cls=LazyEncoder)

    return HttpResponse(result)


@permission_required("core.manage_shop", login_url="/login/")
def save_shipping_method_data(request, shipping_method_id):
    """Saves shipping data (via request body) to the shipping method with passed
    id.

    This is called via an AJAX request and returns JSON encoded data.
    """
    shipping_method = ShippingMethod.objects.get(pk=shipping_method_id)
    shipping_form = ShippingMethodForm(instance=shipping_method, data=request.POST, files=request.FILES)

    form = render_to_string(
        "manage/shipping/shipping_method_data.html", RequestContext(request, {
        "form": shipping_form,
        "shipping_method": shipping_method,
    }))

    if shipping_form.is_valid():
        shipping_form.save()

    return lfs.core.utils.set_message_cookie(
        url=reverse("lfs_manage_shipping_method", kwargs={"shipping_method_id": shipping_method.id}),
        msg=_(u"Shipping method has been saved."),
    )


@require_POST
@permission_required("core.manage_shop", login_url="/login/")
def delete_shipping_method(request, shipping_method_id):
    """Deletes shipping method with passed shipping id.

    All customers, which have selected this shipping method are getting the
    default shipping method.
    """
    try:
        shipping_method = ShippingMethod.objects.get(pk=shipping_method_id)
    except ObjectDoesNotExist:
        pass
    else:
        for customer in Customer.objects.filter(selected_shipping_method=shipping_method_id):
            customer.selected_shipping_method = shipping_utils.get_default_shipping_method(request)
            customer.save()

        shipping_method.delete()

    return lfs.core.utils.set_message_cookie(
        url=reverse("lfs_manage_shipping"),
        msg=_(u"Shipping method has been deleted."),
    )


def _update_price_positions(shipping_method):
    for i, price in enumerate(shipping_method.prices.all()):
        price.priority = (i + 1) * 10
        price.save()



# django imports
from django.contrib.auth.decorators import permission_required
from django.shortcuts import render_to_response
from django.template import RequestContext


@permission_required("core.manage_shop", login_url="/login/")
def dashboard(request, template_name="manage/dashboard.html"):
    """
    """
    return render_to_response(template_name, RequestContext((request)))



# django imports
from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.decorators import permission_required
from django.core.urlresolvers import reverse
from django.db import IntegrityError
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.template.loader import render_to_string
from django.views.decorators.http import require_POST

# lfs imports
from lfs.catalog.models import Product
from lfs.catalog.models import ProductPropertyValue
from lfs.catalog.models import Property
from lfs.catalog.models import PropertyGroup
from lfs.catalog.settings import PROPERTY_NUMBER_FIELD
from lfs.catalog.settings import PROPERTY_TEXT_FIELD
from lfs.catalog.settings import PROPERTY_SELECT_FIELD
from lfs.catalog.settings import PROPERTY_VALUE_TYPE_DEFAULT
from lfs.catalog.settings import PROPERTY_VALUE_TYPE_FILTER
from lfs.catalog.settings import PROPERTY_VALUE_TYPE_DISPLAY
from lfs.core.signals import product_removed_property_group


@permission_required("core.manage_shop", login_url="/login/")
def manage_properties(request, product_id, template_name="manage/product/properties.html"):
    """Displays the UI for manage the properties for the product with passed
    product_id.
    """
    product = get_object_or_404(Product, pk=product_id)

    # Generate lists of properties. For entering values.
    display_configurables = False
    display_filterables = False
    display_displayables = False
    configurables = []
    filterables = []
    displayables = []

    # Configurable
    if not product.is_product_with_variants():
        for property_group in product.property_groups.all():
            properties = []
            for property in property_group.properties.filter(configurable=True).order_by("groupspropertiesrelation"):

                display_configurables = True

                try:
                    ppv = ProductPropertyValue.objects.get(property=property, product=product, type=PROPERTY_VALUE_TYPE_DEFAULT)
                except ProductPropertyValue.DoesNotExist:
                    ppv_id = None
                    ppv_value = ""
                else:
                    ppv_id = ppv.id
                    ppv_value = ppv.value

                # Mark selected options
                options = []
                for option in property.options.all():
                    if str(option.id) == ppv_value:
                        selected = True
                    else:
                        selected = False

                    options.append({
                        "id": option.id,
                        "name": option.name,
                        "selected": selected,
                    })

                properties.append({
                    "id": property.id,
                    "name": property.name,
                    "title": property.title,
                    "type": property.type,
                    "options": options,
                    "display_text_field": property.type in (PROPERTY_TEXT_FIELD, PROPERTY_NUMBER_FIELD),
                    "display_select_field": property.type == PROPERTY_SELECT_FIELD,
                    "value": ppv_value,
                })

            configurables.append({
                "id": property_group.id,
                "name": property_group.name,
                "properties": properties,
            })

        # Filterable
        for property_group in product.property_groups.all():
            properties = []
            for property in property_group.properties.filter(filterable=True).order_by("groupspropertiesrelation"):

                display_filterables = True

                # Try to get the value, if it already exists.
                ppvs = ProductPropertyValue.objects.filter(property=property, product=product, type=PROPERTY_VALUE_TYPE_FILTER)
                value_ids = [ppv.value for ppv in ppvs]

                # Mark selected options
                options = []
                for option in property.options.all():

                    if str(option.id) in value_ids:
                        selected = True
                    else:
                        selected = False

                    options.append({
                        "id": option.id,
                        "name": option.name,
                        "selected": selected,
                    })

                value = ""
                if property.type == PROPERTY_SELECT_FIELD:
                    display_select_field = True
                else:
                    display_select_field = False
                    try:
                        value = value_ids[0]
                    except IndexError:
                        pass

                properties.append({
                    "id": property.id,
                    "name": property.name,
                    "title": property.title,
                    "type": property.type,
                    "options": options,
                    "value" : value,
                    "display_text_field": not display_select_field,
                    "display_select_field": display_select_field,
                })

            filterables.append({
                "id": property_group.id,
                "name": property_group.name,
                "properties": properties,
            })

        # Displayable
        for property_group in product.property_groups.all():
            properties = []
            for property in property_group.properties.filter(display_on_product=True).order_by("groupspropertiesrelation"):

                display_displayables = True

                # Try to get the value, if it already exists.
                ppvs = ProductPropertyValue.objects.filter(property=property, product=product, type=PROPERTY_VALUE_TYPE_DISPLAY)
                value_ids = [ppv.value for ppv in ppvs]

                # Mark selected options
                options = []
                for option in property.options.all():

                    if str(option.id) in value_ids:
                        selected = True
                    else:
                        selected = False

                    options.append({
                        "id": option.id,
                        "name": option.name,
                        "selected": selected,
                    })

                value = ""
                if property.type == PROPERTY_SELECT_FIELD:
                    display_select_field = True
                else:
                    display_select_field = False
                    try:
                        value = value_ids[0]
                    except IndexError:
                        pass

                properties.append({
                    "id": property.id,
                    "name": property.name,
                    "title": property.title,
                    "type": property.type,
                    "options": options,
                    "value" : value,
                    "display_text_field": not display_select_field,
                    "display_select_field": display_select_field,
                })

            displayables.append({
                "id": property_group.id,
                "name": property_group.name,
                "properties": properties,
            })

    # Generate list of all property groups; used for group selection
    product_property_group_ids = [p.id for p in product.property_groups.all()]
    shop_property_groups = []
    for property_group in PropertyGroup.objects.all():

        shop_property_groups.append({
            "id": property_group.id,
            "name": property_group.name,
            "selected": property_group.id in product_property_group_ids,
        })

    return render_to_string(template_name, RequestContext(request, {
        "product": product,
        "filterables": filterables,
        "display_filterables": display_filterables,
        "configurables": configurables,
        "display_configurables": display_configurables,
        "displayables": displayables,
        "display_displayables": display_displayables,
        "product_property_groups": product.property_groups.all(),
        "shop_property_groups": shop_property_groups,
    }))


@require_POST
@permission_required("core.manage_shop", login_url="/login/")
def update_property_groups(request, product_id):
    """Updates property groups for the product with passed id.
    """
    selected_group_ids = request.POST.getlist("selected-property-groups")

    for property_group in PropertyGroup.objects.all():
        # if the group is within selected groups we try to add it to the product
        # otherwise we try do delete it
        if str(property_group.id) in selected_group_ids:
            try:
                property_group.products.get(pk=product_id)
            except ObjectDoesNotExist:
                property_group.products.add(product_id)
        else:
            property_group.products.remove(product_id)
            product = Product.objects.get(pk=product_id)
            product_removed_property_group.send([property_group, product])

    url = reverse("lfs_manage_product", kwargs={"product_id": product_id})
    return HttpResponseRedirect(url)


@require_POST
@permission_required("core.manage_shop", login_url="/login/")
def update_properties(request, product_id):
    """Updates properties for product with passed id.
    """
    type = request.POST.get("type")
    ProductPropertyValue.objects.filter(product=product_id, type=type).delete()

    # Update property values
    for key in request.POST.keys():
        if key.startswith("property") == False:
            continue

        property_id = key.split("-")[1]
        property = get_object_or_404(Property, pk=property_id)
        product = get_object_or_404(Product, pk=product_id)

        for value in request.POST.getlist(key):
            if property.is_valid_value(value):
                ProductPropertyValue.objects.create(product=product, property=property, value=value, type=type)

    url = reverse("lfs_manage_product", kwargs={"product_id": product_id})
    return HttpResponseRedirect(url)



# django imports
from django.db.models.signals import pre_delete, m2m_changed
from django.conf import settings
from django.core.cache import cache

# lfs imports
from lfs.catalog.models import PropertyGroup
from lfs.catalog.models import ProductPropertyValue
from lfs.catalog.models import PropertyOption
from lfs.catalog.models import GroupsPropertiesRelation
from lfs.catalog.models import Category
from lfs.core.signals import property_type_changed
from lfs.core.signals import product_removed_property_group


def property_option_deleted_listener(sender, instance, **kwargs):
    """Deletes all property values which have the deleted PropertyOption
    (instance) selected.

    NOTE: This has to be done via a event/listener pair, because the id of
    the option is stored within value field (a char field) of the Product-
    PropertyValue [1]. That means that there is no relation at ORM level,
    hence the automatic integrity check doesn't take place at all.

    [1] This is done because the value field can also contain free text, for
    instance if the type of the property is a PROPERTY_TEXT_FIELD.
    """
    property = instance.property
    for ppv in ProductPropertyValue.objects.filter(
        property=property, value=str(instance.id)):
        ppv.delete()
pre_delete.connect(property_option_deleted_listener, sender=PropertyOption)


def property_group_deleted_listener(sender, instance, **kwargs):
    """Deletes all ProductPropertyValue, which fullfill following criteria:
       1. Property belongs to the deleted PropertyGroup
       2. Product has the deleted PropertyGroup selected
    """
    properties = instance.properties.all()
    products = instance.products.all()

    for product in products:
        for property in properties:
            try:
                ppv = ProductPropertyValue.objects.get(product=product, property=property)
            except ProductPropertyValue.DoesNotExist:
                pass
            else:
                ppv.delete()
pre_delete.connect(property_group_deleted_listener, sender=PropertyGroup)


def property_removed_listener(sender, instance, **kwargs):
    """This is called when a GroupsPropertiesRelation is about to be deleted.
    This stands for removing a Property from a PropertyGroup.

    This deletes all ProductPropertyValue of products which are in the group of
    question and with the removed property.
    """
    property = instance.property
    products = instance.group.products.all()

    for product in products:
        try:
            ppv = ProductPropertyValue.objects.get(product=product, property=property)
        except ProductPropertyValue.DoesNotExist:
            pass
        else:
            ppv.delete()
pre_delete.connect(property_removed_listener, sender=GroupsPropertiesRelation)


def property_type_changed_listener(sender, **kwargs):
    """This is called when the type of a Property has been changed. Then all
    ProductPropertyValue with this property have to be deleted.
    """
    ppvs = ProductPropertyValue.objects.filter(property=sender)
    ppvs.delete()
property_type_changed.connect(property_type_changed_listener)


def product_removed_from_property_group_listener(sender, **kwargs):
    """
    """
    property_group, product = sender

    # Remove property values for product and properties of the group.
    for property in property_group.properties.all():
        try:
            ppv = ProductPropertyValue.objects.get(product=product, property=property)
        except ProductPropertyValue.DoesNotExist:
            pass
        else:
            ppv.delete()
product_removed_property_group.connect(product_removed_from_property_group_listener)


def product_removed_from_category_listener(sender, instance, action, reverse, model, pk_set, **kwargs):
    if action in [u'post_add', u'post_remove']:
        for pk in pk_set:
            with_parents = False # FIXME: we assume with_parents is False similar to method_signature for get_categories
            cache_key = "%s-product-categories-%s-%s" % (settings.CACHE_MIDDLEWARE_KEY_PREFIX, pk, with_parents)
            categories = cache.delete(cache_key)

m2m_changed.connect(product_removed_from_category_listener, Category.products.through)



# django imports
from django import forms
from django.db import models
from django.template import RequestContext
from django.template.loader import render_to_string

# portlets imports
from portlets.models import Portlet

# lfs imports
import lfs.catalog.utils
import lfs.catalog.models
import lfs.marketing.utils


class TopsellerPortlet(Portlet):
    """Portlet to display recent visited products.
    """
    limit = models.IntegerField(default=5)

    class Meta:
        app_label = 'portlet'

    def __unicode__(self):
        return "%s" % self.id

    def render(self, context):
        """Renders the portlet as html.
        """
        request = context.get("request")
        object = context.get("category") or context.get("product")
        if object is None:
            topseller = lfs.marketing.utils.get_topseller(self.limit)
        elif isinstance(object, lfs.catalog.models.Product):
            category = object.get_current_category(context.get("request"))
            topseller = lfs.marketing.utils.get_topseller_for_category(
                category, self.limit)
        else:
            topseller = lfs.marketing.utils.get_topseller_for_category(
                object, self.limit)

        return render_to_string("lfs/portlets/topseller.html", RequestContext(request, {
            "title": self.title,
            "topseller": topseller,
        }))

    def form(self, **kwargs):
        return TopsellerForm(instance=self, **kwargs)


class TopsellerForm(forms.ModelForm):
    """Form for the TopsellerPortlet.
    """
    class Meta:
        model = TopsellerPortlet



import os
import datetime

from django.conf import settings
from windmill.authoring import djangotest

from lfs.core.models import Shop
from lfs.catalog.models import Product


class CartPriceIntegrationTest(djangotest.WindmillDjangoUnitTest):
    test_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'windmilltests', 'grosspricetests')
    browser = 'chrome'
    settings.TESTING = True
    # We need to set DEBUG = True so that our static content gets server
    settings.DEBUG = True
    fixtures = ['lfs_price_test.xml']

    def setUp(self):
        # Our data is loaded from fixtures
        self.assertEqual(1, Shop.objects.count())

        # Check that apple is using GrossPriceCalculator
        apple = Product.objects.get(slug='apple')
        self.assertEqual('lfs.net_price.NetPriceCalculator', apple.price_calculator)

        # Check that chocolate is using NetPriceCalculator
        chocolate = Product.objects.get(slug='chocolate')
        self.assertEqual('lfs.gross_price.GrossPriceCalculator', chocolate.price_calculator)

        super(CartPriceIntegrationTest, self).setUp()

    def tearDown(self):
        # check for new objects in database
        pass


class ManageInterfaceIntegrationTest(djangotest.WindmillDjangoUnitTest):
    test_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'windmilltests', 'managetests')
    browser = 'chrome'
    settings.TESTING = True
    # We need to set DEBUG = True so that our static content gets server
    settings.DEBUG = True
    fixtures = ['lfs_price_test.xml']

    def setUp(self):
        # Our data is loaded from fixtures
        self.assertEqual(1, Shop.objects.count())

        # Check that apple is using GrossPriceCalculator
        apple = Product.objects.get(slug='apple')
        self.assertEqual('lfs.net_price.NetPriceCalculator', apple.price_calculator)

        super(ManageInterfaceIntegrationTest, self).setUp()

    def tearDown(self):
        # check for new objects in database
        pass



# python imports
import math
import urllib

# django imports
from django.conf import settings
from django.core.cache import cache
from django.core.urlresolvers import reverse
from django.http import Http404
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils import simplejson
from django.utils.translation import ugettext_lazy as _

# lfs imports
import lfs.catalog.utils
import lfs.core.utils
from lfs.caching.utils import lfs_get_object_or_404
from lfs.cart.views import add_to_cart
from lfs.catalog.models import Category
from lfs.catalog.models import File
from lfs.catalog.models import Product
from lfs.catalog.models import ProductPropertyValue
from lfs.catalog.models import PropertyOption
from lfs.catalog.settings import PRODUCT_WITH_VARIANTS
from lfs.catalog.settings import VARIANT
from lfs.catalog.settings import PROPERTY_VALUE_TYPE_DEFAULT
from lfs.catalog.settings import SELECT
from lfs.catalog.settings import CONTENT_PRODUCTS
from lfs.core.utils import LazyEncoder
from lfs.core.templatetags import lfs_tags
from lfs.utils import misc as lfs_utils


def file(request, language=None, id=None):
    """Delivers files to the browser.
    """
    file = lfs_get_object_or_404(File, pk=id)
    response = HttpResponse(file.file, mimetype='application/binary')
    response['Content-Disposition'] = 'attachment; filename=%s' % file.title

    return response


def select_variant(request):
    """This is called via an ajax call if the combination of properties are
    changed.
    """
    variant_id = request.POST.get("variant_id")
    msg = _(u"The product has been changed according to your selection.")

    result = simplejson.dumps({
        "product": product_inline(request, variant_id),
        "message": msg,
    }, cls=LazyEncoder)

    return HttpResponse(result)


def calculate_packing(request, id, quantity=None, as_string=False, template_name="lfs/catalog/packing_result.html"):
    """Calculates the actual amount of pieces to buy on base on packing
    information.
    """
    product = Product.objects.get(pk=id)

    if quantity is None:
        quantity = float(request.POST.get("quantity"))

    packing_amount, packing_unit = product.get_packing_info()

    packs = math.ceil(quantity / packing_amount)
    real_quantity = packs * packing_amount
    price = real_quantity * product.get_price(request)

    html = render_to_string(template_name, RequestContext(request, {
        "price": price,
        "product": product,
        "packs": int(packs),
        "real_quantity": real_quantity,
        "unit": packing_unit,
    }))

    if as_string:
        return html

    result = simplejson.dumps({
        "html": html,
    }, cls=LazyEncoder)

    return HttpResponse(result)


def calculate_price(request, id):
    """Calculates the price of the product on base of choosen properties after
    a customer has selected a property on product view.
    """
    product = Product.objects.get(pk=id)

    property_price = 0
    for key, option_id in request.POST.items():
        if key.startswith("property"):
            try:
                po = PropertyOption.objects.get(pk=option_id)
            except (ValueError, PropertyOption.DoesNotExist):
                pass
            else:
                if po.property.add_price:
                    property_price += po.price

    if product.for_sale:
        for_sale_standard_price = product.get_standard_price(with_properties=False)
        for_sale_standard_price += property_price
    else:
        for_sale_standard_price = 0
        price = product.get_price(request, with_properties=False)
        price += property_price

    result = simplejson.dumps({
        "price": lfs_tags.currency(price),
        "for-sale-standard-price": lfs_tags.currency(for_sale_standard_price),
        "message": _("Price has been changed according to your selection."),
    }, cls=LazyEncoder)

    return HttpResponse(result)


def select_variant_from_properties(request):
    """This is called via an ajax call if the combination of properties are
    changed.
    """
    product_id = request.POST.get("product_id")

    try:
        product = Product.objects.get(pk=product_id)
    except:  # Product.DoesNotExist:
        return HttpResponse("")

    options = lfs_utils.parse_properties(request)
    variant = product.get_variant(options)

    if variant is None:
        msg = _(u"The choosen combination of properties is not deliverable.")
        variant = product.get_default_variant()
    else:
        msg = _(u"The product has been changed according to your selection.")

    result = simplejson.dumps({
        "product": product_inline(request, variant.id),
        "message": msg,
    }, cls=LazyEncoder)

    return HttpResponse(result)


def set_filter(request, category_slug, property_id, value=None, min=None, max=None):
    """Saves the given filter to session. Redirects to the category with given
    slug.
    """
    product_filter = request.session.get("product-filter", {})

    if value is not None:
        product_filter[property_id] = value
    else:
        product_filter[property_id] = (min, max)

    request.session["product-filter"] = product_filter

    url = reverse("lfs_category", kwargs={"slug": category_slug})
    return HttpResponseRedirect(url)


def set_price_filter(request, category_slug):
    """Saves the given price filter to session. Redirects to the category with
    given slug.
    """
    min = request.REQUEST.get("min", "0")
    max = request.REQUEST.get("max", "99999")

    try:
        float(min)
    except TypeError:
        min = "0"

    try:
        float(max)
    except TypeError:
        max = "0"

    request.session["price-filter"] = {"min": min, "max": max}

    url = reverse("lfs_category", kwargs={"slug": category_slug})
    return HttpResponseRedirect(url)


def reset_price_filter(request, category_slug):
    """Resets the price filter. Redirects to the category with given slug.
    """
    if "price-filter" in request.session:
        del request.session["price-filter"]

    url = reverse("lfs_category", kwargs={"slug": category_slug})
    return HttpResponseRedirect(url)


def reset_filter(request, category_slug, property_id):
    """Resets product filter with given property id. Redirects to the category
    with given slug.
    """
    if "product-filter" in request.session:
        if property_id in request.session["product-filter"]:
            del request.session["product-filter"][property_id]
            request.session["product-filter"] = request.session["product-filter"]

    url = reverse("lfs_category", kwargs={"slug": category_slug})
    return HttpResponseRedirect(url)


def reset_all_filter(request, category_slug):
    """Resets all product filter. Redirects to the category with given slug.
    """
    if "product-filter" in request.session:
        del request.session["product-filter"]

    if "price-filter" in request.session:
        del request.session["price-filter"]

    url = reverse("lfs_category", kwargs={"slug": category_slug})
    return HttpResponseRedirect(url)


def set_sorting(request):
    """Saves the given sortings (by request body) to session.
    """
    sorting = request.POST.get("sorting", "")
    if sorting == "" and "sorting" in request.session:
        del request.session["sorting"]
    else:
        request.session["sorting"] = sorting

    # lfs_sorting_changed.send(category_id)
    return HttpResponseRedirect(request.META.get("HTTP_REFERER"))


def category_view(request, slug, template_name="lfs/catalog/category_base.html"):
    """
    """
    start = request.REQUEST.get("start", 0)
    category = lfs_get_object_or_404(Category, slug=slug)
    if category.get_content() == CONTENT_PRODUCTS:
        inline = category_products(request, slug, start)
    else:
        inline = category_categories(request, slug)
    # Set last visited category for later use, e.g. Display breadcrumbs,
    # selected menu points, etc.
    request.session["last_category"] = category

    # TODO: Factor top_category out to a inclusion tag, so that people can
    # omit if they don't need it.

    return render_to_response(template_name, RequestContext(request, {
        "category": category,
        "category_inline": inline,
        "top_category": lfs.catalog.utils.get_current_top_category(request, category),
    }))


def category_categories(request, slug, start=0, template_name="lfs/catalog/categories/category/default.html"):
    """Displays the child categories of the category with passed slug.

    This view is called if the user chooses a template that is situated in settings.CATEGORY_PATH ".
    """
    cache_key = "%s-category-categories-%s" % (settings.CACHE_MIDDLEWARE_KEY_PREFIX, slug)

    result = cache.get(cache_key)
    if result is not None:
        return result

    category = lfs_get_object_or_404(Category, slug=slug)

    format_info = category.get_format_info()
    amount_of_cols = format_info["category_cols"]

    categories = []
    row = []
    for i, children in enumerate(category.get_children()):
        row.append(children)
        if (i + 1) % amount_of_cols == 0:
            categories.append(row)
            row = []

    if len(row) > 0:
        categories.append(row)
    render_template = category.get_template_name()

    if render_template != None:
        template_name = render_template

    result = render_to_string(template_name, RequestContext(request, {
        "category": category,
        "categories": categories,
    }))

    cache.set(cache_key, result)
    return result


def category_products(request, slug, start=0, template_name="lfs/catalog/categories/product/default.html"):
    """Displays the products of the category with passed slug.

    This view is called if the user chooses a template that is situated in settings.PRODUCT_PATH ".
    """
    # Resets the product filters if the user navigates to another category.
    # TODO: Is this what a customer would expect?
    last_category = request.session.get("last_category")
    if (last_category is None) or (last_category.slug != slug):
        if "product-filter" in request.session:
            del request.session["product-filter"]
        if "price-filter" in request.session:
            del request.session["price-filter"]

    try:
        default_sorting = settings.LFS_PRODUCTS_SORTING
    except AttributeError:
        default_sorting = "price"

    sorting = request.session.get("sorting", default_sorting)
    product_filter = request.session.get("product-filter", {})
    product_filter = product_filter.items()

    cache_key = "%s-category-products-%s" % (settings.CACHE_MIDDLEWARE_KEY_PREFIX, slug)
    sub_cache_key = "%s-start-%s-sorting-%s" % (settings.CACHE_MIDDLEWARE_KEY_PREFIX, start, sorting)

    filter_key = ["%s-%s" % (i[0], i[1]) for i in product_filter]
    if filter_key:
        sub_cache_key += "-%s" % "-".join(filter_key)

    price_filter = request.session.get("price-filter")
    if price_filter:
        sub_cache_key += "-%s-%s" % (price_filter["min"], price_filter["max"])

    temp = cache.get(cache_key)
    if temp is not None:
        try:
            return temp[sub_cache_key]
        except KeyError:
            pass
    else:
        temp = dict()

    category = lfs_get_object_or_404(Category, slug=slug)

    # Calculates parameters for display.
    try:
        start = int(start)
    except (ValueError, TypeError):
        start = 0

    format_info = category.get_format_info()
    amount_of_rows = format_info["product_rows"]
    amount_of_cols = format_info["product_cols"]
    amount = amount_of_rows * amount_of_cols

    all_products = lfs.catalog.utils.get_filtered_products_for_category(
        category, product_filter, price_filter, sorting)

    # Calculate products
    row = []
    products = []
    for i, product in enumerate(all_products[start:start + amount]):
        # Switch to default variant if product is a "product with variants".
        # if product.is_product_with_variants() and product.has_variants():
        #    product = product.get_default_variant()
        image = None
        product_image = product.get_image()
        if product_image:
            image = product_image.image
        row.append({
            "obj": product,
            "slug": product.slug,
            "name": product.get_name(),
            "image": image,
            "price": product.get_price(request),
            "standard_price": product.get_standard_price(request),
            "price_unit": product.price_unit,
            "price_includes_tax": product.price_includes_tax,
        })
        if (i + 1) % amount_of_cols == 0:
            products.append(row)
            row = []

    if len(row) > 0:
        products.append(row)

    amount_of_products = all_products.count()

    # Calculate urls
    pages = []
    for i in range(0, amount_of_products / amount + 1):
        page_start = i * amount
        pages.append({
            "name": i + 1,
            "start": page_start,
            "selected": start == page_start,
        })

    if (start + amount) < amount_of_products:
        next_url = "%s?start=%s" % (category.get_absolute_url(), start + amount)
    else:
        next_url = None

    if (start - amount) >= 0:
        previous_url = "%s?start=%s" % (category.get_absolute_url(), start - amount)
    else:
        previous_url = None

    render_template = category.get_template_name()
    if render_template != None:
        template_name = render_template

    result = render_to_string(template_name, RequestContext(request, {
        "category": category,
        "products": products,
        "next_url": next_url,
        "previous_url": previous_url,
        "amount_of_products": amount_of_products,
        "pages": pages,
        "show_pages": amount_of_products > amount,
        "all_products": all_products,
    }))

    temp[sub_cache_key] = result
    cache.set(cache_key, temp)
    return result


def product_view(request, slug, template_name="lfs/catalog/product_base.html"):
    """Main view to display a product.
    """
    product = lfs_get_object_or_404(Product, slug=slug)

    if (request.user.is_superuser or product.is_active()) == False:
        raise Http404()

    # Store recent products for later use
    recent = request.session.get("RECENT_PRODUCTS", [])
    if slug in recent:
        recent.remove(slug)
    recent.insert(0, slug)
    if len(recent) > settings.LFS_RECENT_PRODUCTS_LIMIT:
        recent = recent[:settings.LFS_RECENT_PRODUCTS_LIMIT + 1]
    request.session["RECENT_PRODUCTS"] = recent

    result = render_to_response(template_name, RequestContext(request, {
        "product_inline": product_inline(request, product.id),
        "product": product,
    }))

    return result


def product_inline(request, id, template_name="lfs/catalog/products/product_inline.html"):
    """Part of the prduct view, which displays the actual data of the product.

    This is factored out to be able to better cached and in might in future used
    used to be updated via ajax requests.
    """
    cache_key = "%s-product-inline-%s-%s" % (settings.CACHE_MIDDLEWARE_KEY_PREFIX, request.user.is_superuser, id)
    result = cache.get(cache_key)
    if result is not None:
        return result

    # Get product in question
    product = lfs_get_object_or_404(Product, pk=id)

    if product.sub_type == PRODUCT_WITH_VARIANTS:
        variant = product.get_default_variant()
        if variant is None:
            variant = product
    elif product.sub_type == VARIANT:
        variant = product
        product = product.parent
    else:
        variant = product

    properties = []
    variants = []

    if product.is_product_with_variants():
        if product.variants_display_type == SELECT:
            # Get all properties (sorted). We need to traverse through all
            # property/options to select the options of the current variant.
            for property in product.get_property_select_fields():
                options = []
                for property_option in property.options.all():
                    if variant.has_option(property, property_option):
                        selected = True
                    else:
                        selected = False
                    options.append({
                        "id": property_option.id,
                        "name": property_option.name,
                        "selected": selected,
                    })
                properties.append({
                    "id": property.id,
                    "name": property.name,
                    "title": property.title,
                    "unit": property.unit,
                    "options": options,
                })
        else:
            properties = product.get_property_select_fields()
            variants = product.get_variants()

    elif product.is_configurable_product():
        for property in product.get_configurable_properties():
            options = []

            try:
                ppv = ProductPropertyValue.objects.get(product=product, property=property, type=PROPERTY_VALUE_TYPE_DEFAULT)
                ppv_value = ppv.value
            except ProductPropertyValue.DoesNotExist:
                ppv = None
                ppv_value = ""

            for property_option in property.options.all():
                if ppv_value == str(property_option.id):
                    selected = True
                else:
                    selected = False

                options.append({
                    "id": property_option.id,
                    "name": property_option.name,
                    "price": property_option.price,
                    "selected": selected,
                })

            properties.append({
                "obj": property,
                "id": property.id,
                "name": property.name,
                "title": property.title,
                "unit": property.unit,
                "display_price": property.display_price,
                "options": options,
                "value": ppv_value,
            })

    if product.get_template_name() != None:
        template_name = product.get_template_name()

    if variant.active_packing_unit:
        packing_result = calculate_packing(request, variant.id, 1, True)
    else:
        packing_result = ""

    result = render_to_string(template_name, RequestContext(request, {
        "product": product,
        "standard_price": variant.get_standard_price(request),
        "price": variant.get_price(request),
        "variant": variant,
        "variants": variants,
        "product_accessories": variant.get_accessories(),
        "properties": properties,
        "packing_result": packing_result,
    }))

    cache.set(cache_key, result)
    return result


def product_form_dispatcher(request):
    """Dispatches to the added-to-cart view or to the selected variant.

    This is needed as the product form can have several submit buttons:
       - The add-to-cart button
       - The switch to the selected variant button (only in the case the
         variants of of the product are displayed as select box. This may change
         in future, when the switch may made with an ajax request.)
    """
    if request.REQUEST.get("add-to-cart") is not None:
        return add_to_cart(request)
    else:
        product_id = request.POST.get("product_id")
        product = lfs_get_object_or_404(Product, pk=product_id)

        options = lfs_utils.parse_properties(request)
        variant = product.get_variant(options)

        if variant is None:
            variant = product.get_default_variant()

            return lfs.core.utils.set_message_cookie(
                variant.get_absolute_url(),
                msg=_(u"The choosen combination of properties is not deliverable.")
            )

        return HttpResponseRedirect(variant.get_absolute_url())



# django imports
from django.contrib.contenttypes import generic
from django.db import models
from django.utils.translation import ugettext_lazy as _

from lfs.payment.models import CriteriaObjects
from lfs.catalog.models import DeliveryTime

from lfs.tax.models import Tax


class ActiveShippingMethodManager(models.Manager):
    """A manager which return just active shipping methods.
    """
    def active(self):
        return super(ActiveShippingMethodManager, self).get_query_set().filter(active=True)


class ShippingMethod(models.Model):
    """Decides how bought products are delivered to the customer.

    Instance variables:

    - name
       The name of the shipping method. This is displayed to the customer to
       choose the shipping method.
    - description
       A longer description of the shipping method. This could be displayed to
       the customer to describe the shipping method in detail.
    - note
       This is displayed to the customer within the checkout process and should
       contain a short note about the shipping method.
    - priority
       The order in which the shipping methods are displayed to the customer.
    - image
       An image of the shipping method, which is displayed to customer within
       the checkout process.
    - active
       A flag which decides whether a shipping method is displayed to the
       customer or not.
    - tax
       The tax of the shipping method.
    - price
       The default price of the shipping method. This is taken if the shipping
       method either has no additional prices or if none of he additional prices
       is valid.
    - criteria_objects
       A shipping method may have several criteria which decide whether the
       shipping method is valid. It is valid if all criteria are true. Only
       active and valid shipping methods are provided to the shop customer.

    - delivery_time
       Reference to a delivery_time
    """
    active = models.BooleanField(_(u"Active"), default=False)
    priority = models.IntegerField(_(u"Priority"), default=0)
    name = models.CharField(_(u"Name"), max_length=50)
    description = models.TextField(_(u"Description"), blank=True)
    note = models.TextField(_(u"Note"), blank=True)
    image = models.ImageField(_(u"Image"), upload_to="images", blank=True, null=True)
    tax = models.ForeignKey(Tax, verbose_name=_(u"Tax"), blank=True, null=True)
    price = models.FloatField(_(u"Price"), default=0.0)
    delivery_time = models.ForeignKey(DeliveryTime, verbose_name=_(u"Delivery time"), blank=True, null=True)

    criteria_objects = generic.GenericRelation(CriteriaObjects,
        object_id_field="content_id", content_type_field="content_type")

    objects = ActiveShippingMethodManager()

    class Meta:
        ordering = ("priority", )

    def __unicode__(self):
        return self.name

    def is_valid(self, request, product=None):
        """The shipping method is valid if it has no criteria or if all assigned
        criteria are true.

        If product is given the product is tested otherwise the whole cart.
        """
        from lfs.criteria import utils as criteria_utils
        return criteria_utils.is_valid(self, request, product)


class ShippingMethodPrice(models.Model):
    """An additional price for a shipping method.

    Instance variables:

    - shipping_method
       The shipping method to which the price belongs to.
    - price
       The actual price, which will be billed to the shop customer
    - priority
       The order in which all prices of the belonging shipping method are tested
       for validity. Less comes first.
    - active
       If set to True the shipping price is active. Otherwise it is not active
       and hence not considered with the calculation of the price. Not used at
       the moment within the GUI. Every price is active immediately.
    - criteria_objects
       A shipping method price may have some criteria. Only when all criteria
       are true the price is valid. The first valid price is the actual price of
       the belonging shipping method.
    """

    shipping_method = models.ForeignKey(ShippingMethod, verbose_name=_(u"shipping_method"), related_name="prices")
    price = models.FloatField(_(u"Price"), default=0.0)
    priority = models.IntegerField(_(u"Priority"), default=0)

    # NOTE: At the moment the active attribute is not used within the GUI.
    # We set it to True by default so every added shipping price is active immediately
    active = models.BooleanField(_(u"Active"), default=True)

    criteria_objects = generic.GenericRelation(CriteriaObjects,
        object_id_field="content_id", content_type_field="content_type")

    class Meta:
        ordering = ("priority", )

    def __unicode__(self):
        return "%s" % self.price

    def is_valid(self, request, product=None):
        """The shipping price is valid if it has no criteria or if all assigned
        criteria are true.

        If product is given the product is tested otherwise the whole cart.
        """
        return criteria_utils.is_valid(self, request, product)



# coding: utf-8

from django.utils.translation import gettext_lazy as _
from django.conf import settings

ACTION_PLACE_TABS = 1
ACTION_PLACE_FOOTER = 2

ACTION_PLACE_CHOICES = [
    (ACTION_PLACE_TABS, _(u"Tabs")),
    (ACTION_PLACE_FOOTER, _(u"Footer")),
]
POSTAL_ADDRESS_L10N = getattr(settings, 'POSTAL_ADDRESS_L10N', True)

LFS_DEFAULT_PRICE_CALCULATOR = "lfs.gross_price.GrossPriceCalculator"
LFS_PRICE_CALCULATOR_DICTIONARY = {
    'lfs.gross_price.GrossPriceCalculator': _(u'Price includes tax'),
    'lfs.net_price.NetPriceCalculator': _(u'Price excludes tax'),
}



# django imports
from django.conf import settings
from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test.client import Client
from lfs.catalog.models import Category

from lfs.manage.api.handlers import CategorySortView


class ManageTestCase(TestCase):
    """Tests manage interface
    """
    def setUp(self):
        for i in range(1, 4):
            cat, created = Category.objects.get_or_create(pk=i, name="cat" + str(i), slug="cat" + str(i), position=10, parent=None)

        self.client = Client()

    def test_category_sorting(self):
        """
        Test we get correct sorting of categories from json api
        """

        self.assertEqual(3, Category.objects.count())
        csv = CategorySortView()

        js = 'category[3]=root&category[1]=root&category[2]=1'
        csv.sort_categories(js)
        cat1 = Category.objects.get(pk=1)
        cat2 = Category.objects.get(pk=2)
        cat3 = Category.objects.get(pk=3)

        # check positions are correct
        self.assertEqual(cat1.position, 20)
        self.assertEqual(cat2.position, 30)
        self.assertEqual(cat3.position, 10)

        # check parents are correct
        self.assertEqual(cat1.parent, None)
        self.assertEqual(cat2.parent, cat1)
        self.assertEqual(cat3.parent, None)

        js = 'category[1]=root&category[2]=root&category[3]=2'
        csv.sort_categories(js)
        cat1 = Category.objects.get(pk=1)
        cat2 = Category.objects.get(pk=2)
        cat3 = Category.objects.get(pk=3)

        # check positions are correct
        self.assertEqual(cat1.position, 10)
        self.assertEqual(cat2.position, 20)
        self.assertEqual(cat3.position, 30)

        # check parents are correct
        self.assertEqual(cat1.parent, None)
        self.assertEqual(cat2.parent, None)
        self.assertEqual(cat3.parent, cat2)



# django imports
from django.contrib.auth.decorators import permission_required
from django.core.paginator import EmptyPage
from django.core.paginator import Paginator
from django.db.models import Q
from django.http import HttpResponse
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils import simplejson
from django.utils.translation import ugettext_lazy as _

# lfs.imports
from lfs.caching.utils import lfs_get_object_or_404
from lfs.catalog.models import Category
from lfs.catalog.models import Product
from lfs.catalog.models import ProductAccessories
from lfs.catalog.settings import VARIANT
from lfs.core.signals import product_changed
from lfs.core.utils import LazyEncoder


@permission_required("core.manage_shop", login_url="/login/")
def manage_accessories(request, product_id, template_name="manage/product/accessories.html"):
    """
    """
    product = Product.objects.get(pk=product_id)
    inline = manage_accessories_inline(request, product_id, as_string=True)

    # amount options
    amount_options = []
    for value in (10, 25, 50, 100):
        amount_options.append({
            "value": value,
            "selected": value == request.session.get("accessories-amount")
        })

    return render_to_string(template_name, RequestContext(request, {
        "product": product,
        "accessories_inline": inline,
        "amount_options": amount_options,
    }))


@permission_required("core.manage_shop", login_url="/login/")
def manage_accessories_inline(request, product_id, as_string=False, template_name="manage/product/accessories_inline.html"):
    """View which shows all accessories for the product with the passed id.
    """
    product = Product.objects.get(pk=product_id)
    product_accessories = ProductAccessories.objects.filter(product=product_id)
    accessory_ids = [p.accessory.id for p in product_accessories]

    r = request.REQUEST
    s = request.session

    # If we get the parameter ``keep-filters`` or ``page`` we take the
    # filters out of the request resp. session. The request takes precedence.
    # The page parameter is given if the user clicks on the next/previous page
    # links. The ``keep-filters`` parameters is given is the users adds/removes
    # products. In this way we keeps the current filters when we needed to. If
    # the whole page is reloaded there is no ``keep-filters`` or ``page`` and
    # all filters are reset as they should.

    if r.get("keep-filters") or r.get("page"):
        page = r.get("page", s.get("accessories_page", 1))
        filter_ = r.get("filter", s.get("filter"))
        category_filter = r.get("accessories_category_filter",
                          s.get("accessories_category_filter"))
    else:
        page = r.get("page", 1)
        filter_ = r.get("filter")
        category_filter = r.get("accessories_category_filter")

    # The current filters are saved in any case for later use.
    s["accessories_page"] = page
    s["filter"] = filter_
    s["accessories_category_filter"] = category_filter

    try:
        s["accessories-amount"] = int(r.get("accessories-amount",
                                      s.get("accessories-amount")))
    except TypeError:
        s["accessories-amount"] = 25

    filters = Q()
    if filter_:
        filters &= Q(name__icontains=filter_)
        filters |= Q(sku__icontains=filter_)
        filters |= (Q(sub_type=VARIANT) & Q(active_sku=False) & Q(parent__sku__icontains=filter_))
        filters |= (Q(sub_type=VARIANT) & Q(active_name=False) & Q(parent__name__icontains=filter_))

    if category_filter:
        if category_filter == "None":
            filters &= Q(categories=None)
        elif category_filter == "All":
            pass
        else:
            # First we collect all sub categories and using the `in` operator
            category = lfs_get_object_or_404(Category, pk=category_filter)
            categories = [category]
            categories.extend(category.get_all_children())

            filters &= Q(categories__in=categories)

    products = Product.objects.filter(filters).exclude(pk=product_id)

    paginator = Paginator(products.exclude(pk__in=accessory_ids), s["accessories-amount"])

    try:
        page = paginator.page(page)
    except EmptyPage:
        page = 0

    result = render_to_string(template_name, RequestContext(request, {
        "product": product,
        "product_accessories": product_accessories,
        "page": page,
        "paginator": paginator,
        "filter": filter_,
    }))

    if as_string:
        return result
    else:
        return HttpResponse(
            simplejson.dumps({
                "html": [["#accessories-inline", result]],
            }))


# Actions
@permission_required("core.manage_shop", login_url="/login/")
def load_tab(request, product_id):
    """
    """
    accessories = manage_accessories(request, product_id)
    return HttpResponse(accessories)


@permission_required("core.manage_shop", login_url="/login/")
def add_accessories(request, product_id):
    """Adds passed accessories to product with passed id.
    """
    parent_product = Product.objects.get(pk=product_id)

    for temp_id in request.POST.keys():

        if temp_id.startswith("product") == False:
            continue

        temp_id = temp_id.split("-")[1]
        accessory = Product.objects.get(pk=temp_id)
        product_accessory = ProductAccessories(product=parent_product, accessory=accessory)
        product_accessory.save()

    _update_positions(parent_product)
    product_changed.send(parent_product)

    html = [["#accessories-inline", manage_accessories_inline(request, product_id, as_string=True)]]

    result = simplejson.dumps({
        "html": html,
        "message": _(u"Accessories have been added.")
    }, cls=LazyEncoder)

    return HttpResponse(result)


# TODO: Rename to "update_accessories"
@permission_required("core.manage_shop", login_url="/login/")
def remove_accessories(request, product_id):
    """Removes passed accessories from product with passed id.
    """
    parent_product = Product.objects.get(pk=product_id)

    if request.POST.get("action") == "remove":
        for temp_id in request.POST.keys():

            if temp_id.startswith("accessory") == False:
                continue

            temp_id = temp_id.split("-")[1]
            accessory = Product.objects.get(pk=temp_id)
            product_accessory = ProductAccessories.objects.filter(product=parent_product, accessory=accessory)

            product_accessory.delete()

        _update_positions(parent_product)
        product_changed.send(parent_product)

        html = [["#accessories-inline", manage_accessories_inline(request, product_id, as_string=True)]]

        result = simplejson.dumps({
            "html": html,
            "message": _(u"Accessories have been removed.")
        }, cls=LazyEncoder)

    else:
        for temp_id in request.POST.keys():

            if temp_id.startswith("quantity") == False:
                continue

            temp_id = temp_id.split("-")[1]
            accessory = Product.objects.get(pk=temp_id)
            product_accessory = ProductAccessories.objects.get(product=parent_product, accessory=accessory)

            # Update quantity
            quantity = request.POST.get("quantity-%s" % temp_id)
            product_accessory.quantity = quantity

            # Update position
            position = request.POST.get("position-%s" % temp_id)
            product_accessory.position = position

            product_accessory.save()

            product_changed.send(product_accessory.product)

        _update_positions(parent_product)

        html = [["#accessories-inline", manage_accessories_inline(request, product_id, as_string=True)]]
        result = simplejson.dumps({
            "html": html,
            "message": _(u"Accessories have been updated.")
        }, cls=LazyEncoder)

    return HttpResponse(result)


@permission_required("core.manage_shop", login_url="/login/")
def update_accessories(request, product_id):
    """Updates the accessories activity state for product variants.
    """
    product = Product.objects.get(pk=product_id)
    if request.POST.get("active_accessories"):
        product.active_accessories = True
    else:
        product.active_accessories = False
    product.save()

    html = [["#accessories-inline", manage_accessories_inline(request, product_id, as_string=True)]]
    result = simplejson.dumps({
        "html": html,
        "message": _(u"Accessories have been updated.")
    }, cls=LazyEncoder)

    return HttpResponse(result)


def _update_positions(product):
    """Updates positions of product accessories for given product.
    """
    for i, pa in enumerate(ProductAccessories.objects.filter(product=product)):
        pa.position = (i + 1) * 10
        pa.save()



# django importsad
from django.core.management.base import BaseCommand
from django.db import connection
from django.db import models
from django.utils.translation import ugettext_lazy as _

# lfs imports
import lfs.core.settings as lfs_settings
from lfs.core.utils import get_default_shop
from lfs.voucher.models import Voucher

# south imports
from south.db import db


class Command(BaseCommand):
    args = ''
    help = 'Migrations for LFS'

    def handle(self, *args, **options):
        """
        """
        from lfs.core.models import Application
        from lfs.order.models import Order

        try:
            application = Application.objects.get(pk=1)
        except Application.DoesNotExist:
            application = Application.objects.create(version="0.5")

        version = application.version
        print "Detected version: %s" % version

        # 0.5 -> 0.6
        if version == "0.5":
            print "Migrating to 0.6"

            # Vouchers #######################################################
            db.add_column("voucher_voucher", "used_amount", models.PositiveSmallIntegerField(default=0))
            db.add_column("voucher_voucher", "last_used_date", models.DateTimeField(blank=True, null=True))
            db.add_column("voucher_voucher", "limit", models.PositiveSmallIntegerField(default=1))

            for voucher in Voucher.objects.all():
                voucher.limit = 1
                voucher.save()

            # This mus be done with execute because the old fields are not there
            # anymore (and therefore can't be accessed via attribute) after the user
            # has upgraded to the latest version.
            db.execute("update voucher_voucher set used_amount = 1 where used = 1")
            db.execute("update voucher_voucher set used_amount = 0 where used = 0")
            db.execute("update voucher_voucher set last_used_date = used_date")

            db.delete_column('voucher_voucher', 'used')
            db.delete_column('voucher_voucher', 'used_date')

            # Price calculator ###############################################
            db.add_column("catalog_product", "price_calculator", models.CharField(
                null=True, blank=True, choices=lfs_settings.LFS_PRICE_CALCULATOR_DICTIONARY.items(), max_length=255))

            db.add_column("core_shop", "price_calculator",
                models.CharField(choices=lfs_settings.LFS_PRICE_CALCULATOR_DICTIONARY.items(), default="lfs.gross_price.GrossPriceCalculator", max_length=255))

            # Locale and currency settings
            db.add_column("core_shop", "default_locale",
                models.CharField(_(u"Default Shop Locale"), max_length=20, default="en_US.UTF-8"))
            db.add_column("core_shop", "use_international_currency_code",
                models.BooleanField(_(u"Use international currency codes"), default=False))
            db.delete_column('core_shop', 'default_currency')

            db.add_column("catalog_product", "supplier_id", models.IntegerField("Supplier", blank=True, null=True))

            # Invoice/Shipping countries #####################################
            shop = get_default_shop()
            db.create_table("core_shop_invoice_countries", (
                ("id", models.AutoField(primary_key=True)),
                ("shop_id", models.IntegerField("shop_id")),
                ("country_id", models.IntegerField("country_id")),
            ))
            db.create_index("core_shop_invoice_countries", ("shop_id", ))
            db.create_index("core_shop_invoice_countries", ("country_id", ))
            db.create_unique("core_shop_invoice_countries", ("shop_id", "country_id"))

            db.create_table("core_shop_shipping_countries", (
                ("id", models.AutoField(primary_key=True)),
                ("shop_id", models.IntegerField("shop_id")),
                ("country_id", models.IntegerField("country_id")),
            ))
            db.create_index("core_shop_shipping_countries", ("shop_id", ))
            db.create_index("core_shop_shipping_countries", ("country_id", ))
            db.create_unique("core_shop_shipping_countries", ("shop_id", "country_id"))

            cursor = connection.cursor()
            cursor.execute("""SELECT country_id FROM core_shop_countries""")
            for row in cursor.fetchall():
                shop.invoice_countries.add(row[0])
                shop.shipping_countries.add(row[0])

            db.delete_table("core_shop_countries")

            # Orders #########################################################

            # Add new lines
            db.add_column("order_order", "invoice_line1", models.CharField(null=True, blank=True, max_length=100))
            db.add_column("order_order", "shipping_line1", models.CharField(null=True, blank=True, max_length=100))
            db.add_column("order_order", "invoice_line2", models.CharField(null=True, blank=True, max_length=100))
            db.add_column("order_order", "shipping_line2", models.CharField(null=True, blank=True, max_length=100))
            db.add_column("order_order", "invoice_code", models.CharField(null=True, blank=True, max_length=100))
            db.add_column("order_order", "shipping_code", models.CharField(null=True, blank=True, max_length=100))

            # Migrate data
            cursor.execute("""SELECT id, invoice_zip_code, shipping_zip_code, invoice_street, shipping_street FROM order_order""")
            for row in cursor.fetchall():
                order = Order.objects.get(pk=row[0])
                order.invoice_code = row[1]
                order.shipping_code = row[2]
                order.invoice_line1 = row[3]
                order.shipping_line1 = row[4]
                order.invoice_line2 = ""
                order.shipping_line2 = ""
                order.save()

            # Remove old code fields
            db.delete_column('order_order', 'invoice_zip_code')
            db.delete_column('order_order', 'shipping_zip_code')
            db.delete_column('order_order', 'invoice_street')
            db.delete_column('order_order', 'shipping_street')

            print "Your database has been migrated to version 0.6."

            application.version = "0.6"
            application.save()
        if version == "0.6":
            print "You are up-to-date"



# django imports
from django.contrib import admin

# lfs imports
from lfs.payment.models import PaymentMethod
from lfs.payment.models import PaymentMethodPrice
from lfs.payment.models import PayPalOrderTransaction


class PaymentMethodAdmin(admin.ModelAdmin):
    """
    """
admin.site.register(PaymentMethod, PaymentMethodAdmin)


class PaymentMethodPriceAdmin(admin.ModelAdmin):
    """
    """
admin.site.register(PaymentMethodPrice, PaymentMethodPriceAdmin)


class PayPalOrderTransactionAdmin(admin.ModelAdmin):
    """
    """
admin.site.register(PayPalOrderTransaction, PayPalOrderTransactionAdmin)



from django import forms
from django.utils.safestring import mark_safe


class LFSFileInput(forms.FileInput):
    """A custom file widget which displays the current file.
    """
    def render(self, name, value, attrs=None):
        output = super(LFSFileInput, self).render(name, None, attrs=attrs)
        if value and hasattr(value, "url"):
            output = u"""<div><a href="%s" />%s</a></div>""" % (value.url, value.name) + output

        if value:
            trans = _(u"Delete file")
            output += """<div><input type="checkbox" name="delete_file" id="id_delete_file" /> <label for="delete_file">%s</label></div>""" % trans._proxy____unicode_cast()

        return mark_safe(output)



# django imports
from django.conf import settings

# lfs imports
from lfs.order.models import Order
from lfs.order.settings import PAID, PAYMENT_FAILED, PAYMENT_FLAGGED
from lfs.mail import utils as mail_utils
from models import PayPalOrderTransaction

# other imports
import logging
from paypal.standard.ipn.signals import payment_was_successful, payment_was_flagged
from paypal.standard.pdt.signals import pdt_failed, pdt_successful
from paypal.standard.models import ST_PP_COMPLETED


def mark_payment(pp_obj, order_state=PAID):
    order = None
    try:
        logging.info("getting order for uuid %s" % pp_obj.custom)
        order_uuid = pp_obj.custom
        order = Order.objects.get(uuid=order_uuid)
        if order is not None:
            if order.state != PAID and order_state == PAID:
                if getattr(settings, 'LFS_SEND_ORDER_MAIL_ON_PAYMENT', False):
                    mail_utils.send_order_received_mail(order)
            order.state = order_state
            order.save()
    except Order.DoesNotExist, e:
        logging.error(e)
    return order


def successful_payment(sender, **kwargs):
    logging.info("successful ipn payment")
    ipn_obj = sender
    order = mark_payment(ipn_obj, PAID)
    if order is not None:
        transaction, created = PayPalOrderTransaction.objects.get_or_create(order=order)
        transaction.ipn.add(ipn_obj)
        transaction.save()
    else:
        logging.warning("successful ipn payment, no order found for uuid %s" % ipn_obj.custom)


def unsuccessful_payment(sender, **kwargs):
    logging.info("unsuccessful ipn payment")
    ipn_obj = sender
    if ipn_obj:
        order = None
        if ipn_obj.payment_status == ST_PP_COMPLETED:
            order = mark_payment(ipn_obj, PAYMENT_FLAGGED)
        else:
            order = mark_payment(ipn_obj, PAYMENT_FAILED)
        if order is not None:
            transaction, created = PayPalOrderTransaction.objects.get_or_create(order=order)
            transaction.ipn.add(ipn_obj)
            transaction.save()
        else:
            logging.warning("unsuccessful ipn payment, no order found for uuid %s" % ipn_obj.custom)
    else:
        logging.warning("unsuccessful ipn payment signal with no ipn object")


def successful_pdt(sender, **kwargs):
    logging.info("successful pdt payment")
    pdt_obj = sender
    order = mark_payment(pdt_obj, True)


def unsuccesful_pdt(sender, **kwargs):
    logging.info("unsuccessful pdt payment")
    pdt_obj = sender
    order = mark_payment(pdt_obj, False)


payment_was_successful.connect(successful_payment, dispatch_uid="Order.ipn_successful")
payment_was_flagged.connect(unsuccessful_payment, dispatch_uid="Order.ipn_unsuccessful")
pdt_successful.connect(successful_pdt, dispatch_uid="Order.pdt_successful")
pdt_failed.connect(unsuccesful_pdt, dispatch_uid="Order.pdt_unsuccessful")



# django imports
from django import forms
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.template import RequestContext
from django.template.loader import render_to_string

# portlets imports
from portlets.models import Portlet

# lfs imports
from lfs.catalog.models import Product
from lfs.caching.utils import lfs_get_object


class RecentProductsPortlet(Portlet):
    """Portlet to display recent visited products.
    """
    class Meta:
        app_label = 'portlet'

    def __unicode__(self):
        return "%s" % self.id

    def render(self, context):
        """Renders the portlet as html.
        """
        object = context.get("product")
        slug_not_to_display = ""
        limit = settings.LFS_RECENT_PRODUCTS_LIMIT
        if object:
            ctype = ContentType.objects.get_for_model(object)
            if ctype.name == u"product":
                slug_not_to_display = object.slug
                limit = settings.LFS_RECENT_PRODUCTS_LIMIT + 1

        request = context.get("request")

        products = []
        for slug in request.session.get("RECENT_PRODUCTS", [])[:limit]:
            if slug == slug_not_to_display:
                continue
            product = lfs_get_object(Product, slug=slug)
            if product and product.is_product_with_variants() and product.has_variants():
                product = product.get_default_variant()
            products.append(product)

        return render_to_string("lfs/portlets/recent_products.html", RequestContext(request, {
            "title": self.title,
            "products": products,
        }))

    def form(self, **kwargs):
        return RecentProductsForm(instance=self, **kwargs)


class RecentProductsForm(forms.ModelForm):
    """Form for the RecentProductsPortlet.
    """
    class Meta:
        model = RecentProductsPortlet



# django imports
from django.contrib.auth.decorators import permission_required
from django.core.urlresolvers import reverse
from django.forms import ModelForm
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response
from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _
from django.views.decorators.http import require_POST

# lfs imports
import lfs.core.utils
from lfs.core.models import Action
from lfs.core.models import ActionGroup


class ActionForm(ModelForm):
    """Form to edit an action.
    """
    class Meta:
        model = Action
        exclude = ("parent", "place")


class ActionAddForm(ActionForm):
    """Form to add a action
    """
    class Meta:
        model = Action
        fields = ("title", "group")


@permission_required("core.manage_shop", login_url="/login/")
def manage_actions(request):
    """Dispatches to the first action or to the form to add a action (if there is no
    action yet).
    """
    try:
        action = Action.objects.all()[0]
        url = reverse("lfs_manage_action", kwargs={"id": action.id})
    except IndexError:
        url = reverse("lfs_add_action")

    return HttpResponseRedirect(url)


@permission_required("core.manage_shop", login_url="/login/")
def manage_action(request, id, template_name="manage/shop/action.html"):
    """Provides a form to edit the action with the passed id.
    """
    action = get_object_or_404(Action, pk=id)
    if request.method == "POST":
        form = ActionForm(instance=action, data=request.POST, files=request.FILES)
        if form.is_valid():
            new_action = form.save()
            _update_positions()

            return lfs.core.utils.set_message_cookie(
                url=reverse("lfs_manage_action", kwargs={"id": action.id}),
                msg=_(u"Action has been saved."),
            )
    else:
        form = ActionForm(instance=action)

    return render_to_response(template_name, RequestContext(request, {
        "action": action,
        "groups": ActionGroup.objects.all(),
        "form": form,
        "current_id": int(id),
    }))


@permission_required("core.manage_shop", login_url="/login/")
def add_action(request, template_name="manage/shop/add_action.html"):
    """Provides a form to add a new action.
    """
    if request.method == "POST":
        form = ActionAddForm(data=request.POST, files=request.FILES)
        if form.is_valid():
            action = form.save()
            _update_positions()

            return lfs.core.utils.set_message_cookie(
                url=reverse("lfs_manage_action", kwargs={"id": action.id}),
                msg=_(u"Action has been added."),
            )
    else:
        form = ActionAddForm()

    return render_to_response(template_name, RequestContext(request, {
        "form": form,
        "groups": ActionGroup.objects.all(),
    }))


@require_POST
@permission_required("core.manage_shop", login_url="/login/")
def delete_action(request, id):
    """Deletes the action with passed id.
    """
    action = get_object_or_404(Action, pk=id)
    action.delete()

    return lfs.core.utils.set_message_cookie(
        url=reverse("lfs_manage_actions"),
        msg=_(u"Action has been deleted."),
    )


def _update_positions():
    """Updates the positions of all actions.
    """
    for group in ActionGroup.objects.all():
        for i, action in enumerate(group.actions.all()):
            action.position = (i + 1) * 10
            action.save()



# django imports
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.translation import ugettext_lazy as _

# lfs imports
from lfs.catalog.models import Category
from lfs.catalog.models import Product
from lfs.export.settings import CATEGORY_VARIANTS_CHOICES
from lfs.export.settings import CATEGORY_VARIANTS_DEFAULT


class Export(models.Model):
    """An export of products.
    """
    name = models.CharField(_(u"Name"), max_length=100)
    slug = models.SlugField(_(u"Slug"), unique=True)
    products = models.ManyToManyField(Product, verbose_name=_(u"Products"), blank=True, related_name="exports")
    script = models.ForeignKey("Script", verbose_name=_(u"Script"))
    variants_option = models.PositiveSmallIntegerField(_(u"Variants"), choices=CATEGORY_VARIANTS_CHOICES[1:], default=CATEGORY_VARIANTS_DEFAULT)
    position = models.IntegerField(default=1)

    class Meta:
        ordering = ("position", "name")

    def __unicode__(self):
        return "%s.%s" % (self.module, self.method)

    def get_absolute_url(self):
        """
        """
        return reverse(
            "lfs_export", kwargs={"export_id": self.id})

    def get_products(self):
        """Returns selected products. Takes variant options into account.
        """
        import lfs.export.utils
        products = []

        for product in self.products.all():
            if product.is_product_with_variants():
                variants = lfs.export.utils.get_variants(product, self)
                if variants:
                    products.extend(variants)
            else:
                products.append(product)

        return products


class Script(models.Model):
    """Represents an export script for an Export
    """
    module = models.CharField(max_length=255, default="lfs.export.generic")
    method = models.CharField(max_length=255, default="export")
    name = models.CharField(max_length=100, unique=True)

    def __unicode__(self):
        return self.name

    class Meta:
        ordering = ("name", )
        unique_together = ("module", "method")


class CategoryOption(models.Model):
    """Stores options for categories.
    """
    category = models.ForeignKey(Category, verbose_name=_(u"Category"))
    export = models.ForeignKey(Export, verbose_name=_(u"Export"))
    variants_option = models.PositiveSmallIntegerField(_(u"Variant"), choices=CATEGORY_VARIANTS_CHOICES)



# django imports
from django.contrib.auth.decorators import permission_required
from django.forms import ModelForm
from django.http import HttpResponse
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils import simplejson
from django.utils.translation import ugettext_lazy as _

# lfs.imports
from lfs.caching.utils import lfs_get_object_or_404
from lfs.catalog.models import Product
from lfs.core.utils import LazyEncoder


class SEOForm(ModelForm):
    """Form to add/edit seo properties of a product.
    """
    class Meta:
        model = Product
        fields = (
            "active_meta_title", "meta_title",
            "active_meta_keywords", "meta_keywords",
            "active_meta_description", "meta_description",
        )


@permission_required("core.manage_shop", login_url="/login/")
def manage_seo(request, product_id, template_name="manage/product/seo.html"):
    """
    """
    product = lfs_get_object_or_404(Product, pk=product_id)

    if request.method == "POST":
        form = SEOForm(instance=product, data=request.POST)
        if form.is_valid():
            form.save()
    else:
        form = SEOForm(instance=product)

    result = render_to_string(template_name, RequestContext(request, {
        "product": product,
        "form": form,
    }))

    if request.is_ajax():
        return HttpResponse(simplejson.dumps({
            "message": _(u"Seo data has been saved."),
        }, cls=LazyEncoder))
    else:
        return result



# django imports
from django import forms
from django.conf import settings
from django.core.cache import cache
from django.template import RequestContext
from django.template.loader import render_to_string

# portlets imports
from portlets.models import Portlet

# lfs imports
from lfs.page.models import Page


class PagesPortlet(Portlet):
    """Portlet to display pages.
    """
    class Meta:
        app_label = 'portlet'

    def __unicode__(self):
        return "%s" % self.id

    def render(self, context):
        """Renders the portlet as html.
        """
        request = context.get("request")

        cache_key = "%s-pages" % settings.CACHE_MIDDLEWARE_KEY_PREFIX
        pages = cache.get(cache_key)
        if pages is None:
            pages = Page.objects.filter(active=True, exclude_from_navigation=False)
            cache.set(cache_key, pages)

        return render_to_string("lfs/portlets/pages.html", RequestContext(request, {
            "title": self.title,
            "pages": pages,
        }))

    def form(self, **kwargs):
        return PagesForm(instance=self, **kwargs)


class PagesForm(forms.ModelForm):
    """Form for the PagesPortlet.
    """
    class Meta:
        model = PagesPortlet



# django imports
from django.contrib import admin

# lfs imports
from lfs.customer.models import Customer


class CustomerAdmin(admin.ModelAdmin):
    """
    """
admin.site.register(Customer, CustomerAdmin)



# django imports
from django.contrib import admin

# lfs imports
from lfs.supplier.models import Supplier


class SupplierAdmin(admin.ModelAdmin):
    """
    """
admin.site.register(Supplier, SupplierAdmin)



# django imports
from django.contrib.auth.models import User
from django.contrib.auth.models import AnonymousUser
from django.contrib.sessions.backends.file import SessionStore
from django.shortcuts import get_object_or_404
from django.test import TestCase
from django.test.client import Client

# test imports
from lfs.catalog.models import Product
from lfs.cart.models import Cart
from lfs.cart.models import CartItem
from lfs.cart.views import add_to_cart
from lfs.cart import utils as cart_utils
from lfs.core.models import Country
from lfs.customer.models import Address
from lfs.customer.models import Customer
from lfs.order.models import Order
from lfs.order.utils import add_order
from lfs.order.settings import SUBMITTED
from lfs.payment.models import PaymentMethod
from lfs.shipping.models import ShippingMethod
from lfs.tax.models import Tax
from lfs.tests.utils import DummySession
from lfs.tests.utils import RequestFactory


class OrderTestCase(TestCase):
    """
    """
    fixtures = ['lfs_shop.xml']

    def setUp(self):
        """
        """
        session = SessionStore()

        rf = RequestFactory()
        self.request = rf.get('/')
        self.request.session = session
        self.request.user = AnonymousUser()

        tax = Tax.objects.create(rate=19)

        shipping_method = ShippingMethod.objects.create(
            name="Standard",
            active=True,
            price=1.0,
            tax=tax
        )

        payment_method = PaymentMethod.objects.create(
            name="Direct Debit",
            active=True,
            tax=tax,
        )

        us = Country.objects.get(code="us")
        ie = Country.objects.get(code="ie")

        address1 = Address.objects.create(
            firstname="John",
            lastname="Doe",
            company_name="Doe Ltd.",
            street="Street 42",
            city="Gotham City",
            zip_code="2342",
            country=ie,
            phone="555-111111",
            email="john@doe.com",
        )

        address2 = Address.objects.create(
            firstname="Jane",
            lastname="Doe",
            company_name="Doe Ltd.",
            street="Street 43",
            city="Smallville",
            zip_code="2443",
            country=us,
            phone="666-111111",
            email="jane@doe.com",
        )

        self.customer = Customer.objects.create(
            session=session.session_key,
            selected_shipping_method=shipping_method,
            selected_payment_method=payment_method,
            selected_shipping_address=address1,
            selected_invoice_address=address2,
        )

        p1 = Product.objects.create(
            name="Product 1",
            slug="product-1",
            sku="sku-1",
            price=1.1,
            tax=tax,
        )

        p2 = Product.objects.create(
            name="Product 2",
            slug="product-2",
            sku="sku-2",
            price=2.2,
            tax=tax,
        )

        cart = Cart.objects.create(
            session=session.session_key
        )

        item = CartItem.objects.create(
            cart=cart,
            product=p1,
            amount=2,
        )

        item = CartItem.objects.create(
            cart=cart,
            product=p2,
            amount=3,
        )

    def test_add_order(self):
        """Tests the general adding of an order via the add_order method
        """
        order = add_order(self.request)

        self.assertEqual(order.state, SUBMITTED)
        self.assertEqual("%.2f" % order.price, "9.80")
        self.assertEqual("%.2f" % order.tax, "1.56")

        self.assertEqual(order.shipping_method.name, "Standard")
        self.assertEqual(order.shipping_price, 1.0)
        self.assertEqual("%.2f" % order.shipping_tax, "0.16")

        self.assertEqual(order.payment_method.name, "Direct Debit")
        self.assertEqual(order.payment_price, 0.0)
        self.assertEqual(order.payment_tax, 0.0)

        self.assertEqual(order.shipping_firstname, "John")
        self.assertEqual(order.shipping_lastname, "Doe")
        self.assertEqual(order.shipping_line1, "Doe Ltd.")
        self.assertEqual(order.shipping_line2, "Street 42")
        self.assertEqual(order.shipping_city, "Gotham City")
        self.assertEqual(order.shipping_code, "2342")
        self.assertEqual(order.shipping_phone, "555-111111")

        self.assertEqual(order.invoice_firstname, "Jane")
        self.assertEqual(order.invoice_lastname, "Doe")
        self.assertEqual(order.invoice_line1, "Doe Ltd.")
        self.assertEqual(order.invoice_line2, "Street 43")
        self.assertEqual(order.invoice_city, "Smallville")
        self.assertEqual(order.invoice_code, "2443")
        self.assertEqual(order.invoice_phone, "666-111111")

        # Items
        self.assertEqual(len(order.items.all()), 2)

        item = order.items.all().order_by('id')[0]
        self.assertEqual(item.product_amount, 2)
        self.assertEqual(item.product_sku, "sku-1")
        self.assertEqual(item.product_name, "Product 1")
        self.assertEqual("%.2f" % item.product_price_gross, "1.10")
        self.assertEqual("%.2f" % item.product_price_net, "0.92")
        self.assertEqual("%.2f" % item.product_tax, "0.18")

        item = order.items.all().order_by('id')[1]
        self.assertEqual(item.product_amount, 3)
        self.assertEqual(item.product_sku, "sku-2")
        self.assertEqual(item.product_name, "Product 2")
        self.assertEqual("%.2f" % item.product_price_gross, "2.20")
        self.assertEqual("%.2f" % item.product_price_net, "1.85")
        self.assertEqual("%.2f" % item.product_tax, "0.35")

        # The cart should be deleted after the order has been created
        cart = cart_utils.get_cart(self.request)
        self.assertEqual(cart, None)

    def test_pay_link(self):
        """Tests empty pay link.
        """
        from lfs.payment.utils import process_payment
        result = process_payment(self.request)

        order = Order.objects.filter()[0]
        self.assertEqual(order.get_pay_link(), "")

    def test_paypal_link(self):
        """Tests created paypal link.
        """
        payment_method, created = PaymentMethod.objects.get_or_create(
            id=3,
            name="PayPal",
            active=True,
        )

        self.customer.selected_payment_method = payment_method
        self.customer.save()

        from lfs.payment.utils import process_payment
        result = process_payment(self.request)

        order = Order.objects.filter()[0]
        self.failIf(order.get_pay_link().find("paypal") == -1)



# django imports
from django.conf.urls.defaults import *
from django.views.generic.simple import direct_to_template

# lfs imports
from lfs.core.sitemap import ProductSitemap
from lfs.core.sitemap import CategorySitemap
from lfs.core.views import one_time_setup

# Robots
urlpatterns = patterns('django.views.generic.simple',
    (r'^robots.txt', 'direct_to_template', {'template': 'lfs/shop/robots.txt'}),
)

# Sitemaps
urlpatterns += patterns("django.contrib.sitemaps.views",
    url(r'^sitemap.xml$', 'sitemap', {'sitemaps': {"products": ProductSitemap, "categories": CategorySitemap}})
)

# Shop
urlpatterns += patterns('lfs.core.views',
    url(r'^$', "shop_view", name="lfs_shop_view"),
)

# Cart
urlpatterns += patterns('lfs.cart.views',
    url(r'^add-to-cart$', "add_to_cart"),
    url(r'^add-accessory-to-cart/(?P<product_id>\d*)/(?P<quantity>.*)$', "add_accessory_to_cart", name="lfs_add_accessory_to_cart"),
    url(r'^added-to-cart$', "added_to_cart", name="lfs_added_to_cart"),
    url(r'^delete-cart-item/(?P<cart_item_id>\d*)$', "delete_cart_item", name="lfs_delete_cart_item"),
    url(r'^refresh-cart$', "refresh_cart"),
    url(r'^cart$', "cart", name="lfs_cart"),
    url(r'^check-voucher-cart/$', "check_voucher", name="lfs_check_voucher_cart"),
)

# Catalog
urlpatterns += patterns('lfs.catalog.views',
    url(r'^category-(?P<slug>[-\w]*)$', "category_view", name="lfs_category"),
    url(r'^product/(?P<slug>[-\w]*)$', "product_view", name="lfs_product"),
    url(r'^product-form-dispatcher', "product_form_dispatcher", name="lfs_product_dispatcher"),
    url(r'^set-sorting', "set_sorting", name="lfs_catalog_set_sorting"),
    url(r'^set-product-filter/(?P<category_slug>[-\w]+)/(?P<property_id>\d+)/(?P<min>.+)/(?P<max>.+)', "set_filter", name="lfs_set_product_filter"),
    url(r'^set-product-filter/(?P<category_slug>[-\w]+)/(?P<property_id>\d+)/(?P<value>.+)', "set_filter", name="lfs_set_product_filter"),
    url(r'^set-price-filter/(?P<category_slug>[-\w]+)/$', "set_price_filter", name="lfs_set_price_filter"),
    url(r'^reset-price-filter/(?P<category_slug>[-\w]+)/$', "reset_price_filter", name="lfs_reset_price_filter"),
    url(r'^reset-product-filter/(?P<category_slug>[-\w]+)/(?P<property_id>\d+)', "reset_filter", name="lfs_reset_product_filter"),
    url(r'^reset-all-product-filter/(?P<category_slug>[-\w]+)', "reset_all_filter", name="lfs_reset_all_product_filter"),
    url(r'^select-variant$', "select_variant", name="lfs_select_variant"),
    url(r'^select-variant-from-properties$', "select_variant_from_properties", name="lfs_select_variant_from_properties"),
    url(r'^file/(?P<id>[-\w]*)', "file", name="lfs_file"),
    url(r'^calculate-price/(?P<id>[-\w]*)', "calculate_price", name="lfs_calculate_price"),
    url(r'^calculate-packing/(?P<id>[-\w]*)', "calculate_packing", name="lfs_calculate_packing"),
)

# Checkout
urlpatterns += patterns('lfs.checkout.views',
    url(r'^checkout-dispatcher', "checkout_dispatcher", name="lfs_checkout_dispatcher"),
    url(r'^checkout-login', "login", name="lfs_checkout_login"),
    url(r'^checkout', "one_page_checkout", name="lfs_checkout"),
    url(r'^thank-you', "thank_you", name="lfs_thank_you"),
    url(r'^changed-checkout/$', "changed_checkout", name="lfs_changed_checkout"),
    url(r'^changed-invoice-country/$', "changed_invoice_country", name="lfs_changed_invoice_country"),
    url(r'^changed-shipping-country/$', "changed_shipping_country", name="lfs_changed_shipping_country"),
    url(r'^check-voucher/$', "check_voucher", name="lfs_check_voucher"),
)

# Customer
urlpatterns += patterns('lfs.customer.views',
    url(r'^login', "login", name="lfs_login"),
    url(r'^logout', "logout", name="lfs_logout"),
    url(r'^my-account', "account", name="lfs_my_account"),
    url(r'^my-addresses', "addresses", name="lfs_my_addresses"),
    url(r'^my-email', "email", name="lfs_my_email"),
    url(r'^my-orders', "orders", name="lfs_my_orders"),
    url(r'^my-order/(?P<id>\d+)', "order", name="lfs_my_order"),
    url(r'^my-password', "password", name="lfs_my_password"),
)

# Page
urlpatterns += patterns('lfs.page.views',
    url(r'^page/(?P<slug>[-\w]*)$', "page_view", name="lfs_page_view"),
    url(r'^pages/$', "pages_view", name="lfs_pages"),
    url(r'^popup/(?P<slug>[-\w]*)$', "popup_view", name="lfs_popup_view"),
)

# Password reset
urlpatterns += patterns('django.contrib.auth.views',
     url(r'^password-reset/$', "password_reset", name="lfs_password_reset"),
     url(r'^password-reset-done/$', "password_reset_done"),
     url(r'^password-reset-confirm/(?P<uidb36>[-\w]*)/(?P<token>[-\w]*)$', "password_reset_confirm"),
     url(r'^password-reset-complete/$', "password_reset_complete"),
)

# Search
urlpatterns += patterns('lfs.search.views',
    url(r'^search', "search", name="lfs_search"),
    url(r'^livesearch', "livesearch", name="lfs_livesearch"),
)

# Tagging
urlpatterns += patterns('',
    (r'^tagging/', include('lfs.tagging.urls')),
)

# Catalog REST API
urlpatterns += patterns('',
    (r'^catalog/api/', include('lfs.catalog.api.urls')),
)

# Manage REST API
urlpatterns += patterns('',
    (r'^manage/api/', include('lfs.manage.api.urls')),
)

urlpatterns += patterns('',
    (r'', include('lfs_contact.urls')),
)


one_time_setup()



# django imports
from django.conf import settings
from django import forms
from django.core.cache import cache
from django.db import models
from django.template import RequestContext
from django.template.loader import render_to_string

# portlets
from portlets.models import Portlet

# lfs imports
import lfs.core.utils


class CategoriesPortlet(Portlet):
    """Portlet to display categories.
    """
    start_level = models.PositiveSmallIntegerField(default=1)
    expand_level = models.PositiveSmallIntegerField(default=1)

    class Meta:
        app_label = 'portlet'

    def __unicode__(self):
        return "%s" % self.id

    def render(self, context):
        """Renders the portlet as html.
        """
        # Calculate current categories
        request = context.get("request")

        product = context.get("product")
        category = context.get("category")
        object = category or product

        if object is None:
            object_id = None
        else:
            object_id = object.id

        cache_key = "%s-categories-portlet-%s-%s" % (settings.CACHE_MIDDLEWARE_KEY_PREFIX, object.__class__.__name__, object_id)
        result = cache.get(cache_key)
        if result is not None:
            return result

        current_categories = lfs.core.utils.get_current_categories(request, object)

        ct = lfs.core.utils.CategoryTree(
            current_categories, self.start_level, self.expand_level)
        category_tree = ct.get_category_tree()

        result = render_to_string("lfs/portlets/categories.html", RequestContext(request, {
            "title": self.title,
            "categories": category_tree,
            "product": product,
            "category": category,
        }))

        cache.set(cache_key, result)
        return result

    def form(self, **kwargs):
        return CategoriesPortletForm(instance=self, **kwargs)


class CategoriesPortletForm(forms.ModelForm):
    """Form for CategoriesPortlet.
    """
    class Meta:
        model = CategoriesPortlet
